# coding: utf-8

"""
    Cherwell REST API

    Unofficial Python Cherwell REST API library.  # noqa: E501

    The version of the OpenAPI document: 9.3.2
    Contact: See AUTHORS.
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from pycherwell.configuration import Configuration


class FieldTemplateItem(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'dirty': 'bool',
        'display_name': 'str',
        'field_id': 'str',
        'html': 'str',
        'name': 'str',
        'value': 'str'
    }

    attribute_map = {
        'dirty': 'dirty',
        'display_name': 'displayName',
        'field_id': 'fieldId',
        'html': 'html',
        'name': 'name',
        'value': 'value'
    }

    def __init__(self, dirty=None, display_name=None, field_id=None, html=None, name=None, value=None, local_vars_configuration=None):  # noqa: E501
        """FieldTemplateItem - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._dirty = None
        self._display_name = None
        self._field_id = None
        self._html = None
        self._name = None
        self._value = None
        self.discriminator = None

        if dirty is not None:
            self.dirty = dirty
        if display_name is not None:
            self.display_name = display_name
        if field_id is not None:
            self.field_id = field_id
        if html is not None:
            self.html = html
        if name is not None:
            self.name = name
        if value is not None:
            self.value = value

    @property
    def dirty(self):
        """Gets the dirty of this FieldTemplateItem.  # noqa: E501


        :return: The dirty of this FieldTemplateItem.  # noqa: E501
        :rtype: bool
        """
        return self._dirty

    @dirty.setter
    def dirty(self, dirty):
        """Sets the dirty of this FieldTemplateItem.


        :param dirty: The dirty of this FieldTemplateItem.  # noqa: E501
        :type: bool
        """

        self._dirty = dirty

    @property
    def display_name(self):
        """Gets the display_name of this FieldTemplateItem.  # noqa: E501


        :return: The display_name of this FieldTemplateItem.  # noqa: E501
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name):
        """Sets the display_name of this FieldTemplateItem.


        :param display_name: The display_name of this FieldTemplateItem.  # noqa: E501
        :type: str
        """

        self._display_name = display_name

    @property
    def field_id(self):
        """Gets the field_id of this FieldTemplateItem.  # noqa: E501


        :return: The field_id of this FieldTemplateItem.  # noqa: E501
        :rtype: str
        """
        return self._field_id

    @field_id.setter
    def field_id(self, field_id):
        """Sets the field_id of this FieldTemplateItem.


        :param field_id: The field_id of this FieldTemplateItem.  # noqa: E501
        :type: str
        """

        self._field_id = field_id

    @property
    def html(self):
        """Gets the html of this FieldTemplateItem.  # noqa: E501


        :return: The html of this FieldTemplateItem.  # noqa: E501
        :rtype: str
        """
        return self._html

    @html.setter
    def html(self, html):
        """Sets the html of this FieldTemplateItem.


        :param html: The html of this FieldTemplateItem.  # noqa: E501
        :type: str
        """

        self._html = html

    @property
    def name(self):
        """Gets the name of this FieldTemplateItem.  # noqa: E501


        :return: The name of this FieldTemplateItem.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this FieldTemplateItem.


        :param name: The name of this FieldTemplateItem.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def value(self):
        """Gets the value of this FieldTemplateItem.  # noqa: E501


        :return: The value of this FieldTemplateItem.  # noqa: E501
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this FieldTemplateItem.


        :param value: The value of this FieldTemplateItem.  # noqa: E501
        :type: str
        """

        self._value = value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, FieldTemplateItem):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, FieldTemplateItem):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Cherwell REST API

    Unofficial Python Cherwell REST API library.  # noqa: E501

    The version of the OpenAPI document: 9.3.2
    Contact: See AUTHORS.
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from pycherwell.configuration import Configuration


class SaveApiClientSettingRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'allow_anonymous_access': 'bool',
        'api_access_is_enabled': 'bool',
        'create_new_client_key': 'bool',
        'culture': 'str',
        'description': 'str',
        'name': 'str',
        'refresh_token_lifespan_minutes': 'int',
        'stand_in_key': 'str',
        'token_lifespan_minutes': 'int'
    }

    attribute_map = {
        'allow_anonymous_access': 'allowAnonymousAccess',
        'api_access_is_enabled': 'apiAccessIsEnabled',
        'create_new_client_key': 'createNewClientKey',
        'culture': 'culture',
        'description': 'description',
        'name': 'name',
        'refresh_token_lifespan_minutes': 'refreshTokenLifespanMinutes',
        'stand_in_key': 'standInKey',
        'token_lifespan_minutes': 'tokenLifespanMinutes'
    }

    def __init__(self, allow_anonymous_access=None, api_access_is_enabled=None, create_new_client_key=None, culture=None, description=None, name=None, refresh_token_lifespan_minutes=None, stand_in_key=None, token_lifespan_minutes=None, local_vars_configuration=None):  # noqa: E501
        """SaveApiClientSettingRequest - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._allow_anonymous_access = None
        self._api_access_is_enabled = None
        self._create_new_client_key = None
        self._culture = None
        self._description = None
        self._name = None
        self._refresh_token_lifespan_minutes = None
        self._stand_in_key = None
        self._token_lifespan_minutes = None
        self.discriminator = None

        if allow_anonymous_access is not None:
            self.allow_anonymous_access = allow_anonymous_access
        if api_access_is_enabled is not None:
            self.api_access_is_enabled = api_access_is_enabled
        if create_new_client_key is not None:
            self.create_new_client_key = create_new_client_key
        if culture is not None:
            self.culture = culture
        if description is not None:
            self.description = description
        if name is not None:
            self.name = name
        if refresh_token_lifespan_minutes is not None:
            self.refresh_token_lifespan_minutes = refresh_token_lifespan_minutes
        if stand_in_key is not None:
            self.stand_in_key = stand_in_key
        if token_lifespan_minutes is not None:
            self.token_lifespan_minutes = token_lifespan_minutes

    @property
    def allow_anonymous_access(self):
        """Gets the allow_anonymous_access of this SaveApiClientSettingRequest.  # noqa: E501


        :return: The allow_anonymous_access of this SaveApiClientSettingRequest.  # noqa: E501
        :rtype: bool
        """
        return self._allow_anonymous_access

    @allow_anonymous_access.setter
    def allow_anonymous_access(self, allow_anonymous_access):
        """Sets the allow_anonymous_access of this SaveApiClientSettingRequest.


        :param allow_anonymous_access: The allow_anonymous_access of this SaveApiClientSettingRequest.  # noqa: E501
        :type: bool
        """

        self._allow_anonymous_access = allow_anonymous_access

    @property
    def api_access_is_enabled(self):
        """Gets the api_access_is_enabled of this SaveApiClientSettingRequest.  # noqa: E501


        :return: The api_access_is_enabled of this SaveApiClientSettingRequest.  # noqa: E501
        :rtype: bool
        """
        return self._api_access_is_enabled

    @api_access_is_enabled.setter
    def api_access_is_enabled(self, api_access_is_enabled):
        """Sets the api_access_is_enabled of this SaveApiClientSettingRequest.


        :param api_access_is_enabled: The api_access_is_enabled of this SaveApiClientSettingRequest.  # noqa: E501
        :type: bool
        """

        self._api_access_is_enabled = api_access_is_enabled

    @property
    def create_new_client_key(self):
        """Gets the create_new_client_key of this SaveApiClientSettingRequest.  # noqa: E501


        :return: The create_new_client_key of this SaveApiClientSettingRequest.  # noqa: E501
        :rtype: bool
        """
        return self._create_new_client_key

    @create_new_client_key.setter
    def create_new_client_key(self, create_new_client_key):
        """Sets the create_new_client_key of this SaveApiClientSettingRequest.


        :param create_new_client_key: The create_new_client_key of this SaveApiClientSettingRequest.  # noqa: E501
        :type: bool
        """

        self._create_new_client_key = create_new_client_key

    @property
    def culture(self):
        """Gets the culture of this SaveApiClientSettingRequest.  # noqa: E501


        :return: The culture of this SaveApiClientSettingRequest.  # noqa: E501
        :rtype: str
        """
        return self._culture

    @culture.setter
    def culture(self, culture):
        """Sets the culture of this SaveApiClientSettingRequest.


        :param culture: The culture of this SaveApiClientSettingRequest.  # noqa: E501
        :type: str
        """

        self._culture = culture

    @property
    def description(self):
        """Gets the description of this SaveApiClientSettingRequest.  # noqa: E501


        :return: The description of this SaveApiClientSettingRequest.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this SaveApiClientSettingRequest.


        :param description: The description of this SaveApiClientSettingRequest.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def name(self):
        """Gets the name of this SaveApiClientSettingRequest.  # noqa: E501


        :return: The name of this SaveApiClientSettingRequest.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this SaveApiClientSettingRequest.


        :param name: The name of this SaveApiClientSettingRequest.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def refresh_token_lifespan_minutes(self):
        """Gets the refresh_token_lifespan_minutes of this SaveApiClientSettingRequest.  # noqa: E501


        :return: The refresh_token_lifespan_minutes of this SaveApiClientSettingRequest.  # noqa: E501
        :rtype: int
        """
        return self._refresh_token_lifespan_minutes

    @refresh_token_lifespan_minutes.setter
    def refresh_token_lifespan_minutes(self, refresh_token_lifespan_minutes):
        """Sets the refresh_token_lifespan_minutes of this SaveApiClientSettingRequest.


        :param refresh_token_lifespan_minutes: The refresh_token_lifespan_minutes of this SaveApiClientSettingRequest.  # noqa: E501
        :type: int
        """

        self._refresh_token_lifespan_minutes = refresh_token_lifespan_minutes

    @property
    def stand_in_key(self):
        """Gets the stand_in_key of this SaveApiClientSettingRequest.  # noqa: E501


        :return: The stand_in_key of this SaveApiClientSettingRequest.  # noqa: E501
        :rtype: str
        """
        return self._stand_in_key

    @stand_in_key.setter
    def stand_in_key(self, stand_in_key):
        """Sets the stand_in_key of this SaveApiClientSettingRequest.


        :param stand_in_key: The stand_in_key of this SaveApiClientSettingRequest.  # noqa: E501
        :type: str
        """

        self._stand_in_key = stand_in_key

    @property
    def token_lifespan_minutes(self):
        """Gets the token_lifespan_minutes of this SaveApiClientSettingRequest.  # noqa: E501


        :return: The token_lifespan_minutes of this SaveApiClientSettingRequest.  # noqa: E501
        :rtype: int
        """
        return self._token_lifespan_minutes

    @token_lifespan_minutes.setter
    def token_lifespan_minutes(self, token_lifespan_minutes):
        """Sets the token_lifespan_minutes of this SaveApiClientSettingRequest.


        :param token_lifespan_minutes: The token_lifespan_minutes of this SaveApiClientSettingRequest.  # noqa: E501
        :type: int
        """

        self._token_lifespan_minutes = token_lifespan_minutes

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SaveApiClientSettingRequest):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SaveApiClientSettingRequest):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Cherwell REST API

    Unofficial Python Cherwell REST API library.  # noqa: E501

    The version of the OpenAPI document: 9.3.2
    Contact: See AUTHORS.
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from pycherwell.configuration import Configuration


class QuickSearchConfigurationResponse(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'allow_quick_search': 'bool',
        'allow_specific_search': 'bool',
        'default_to_quick_search': 'bool',
        'display_name': 'str',
        'gallery_image': 'str',
        'history': 'list[str]',
        'include_available_in_specific': 'bool',
        'include_quick_search_in_specific': 'bool',
        'quick_search_id': 'str',
        'quick_search_items': 'list[QuickSearchItem]',
        'quick_search_watermark': 'str',
        'resolved_quick_search_watermark': 'str',
        'scope': 'str',
        'scope_owner': 'str',
        'specific_search_items': 'list[QuickSearchItem]',
        'error_code': 'str',
        'error_message': 'str',
        'has_error': 'bool'
    }

    attribute_map = {
        'allow_quick_search': 'allowQuickSearch',
        'allow_specific_search': 'allowSpecificSearch',
        'default_to_quick_search': 'defaultToQuickSearch',
        'display_name': 'displayName',
        'gallery_image': 'galleryImage',
        'history': 'history',
        'include_available_in_specific': 'includeAvailableInSpecific',
        'include_quick_search_in_specific': 'includeQuickSearchInSpecific',
        'quick_search_id': 'quickSearchId',
        'quick_search_items': 'quickSearchItems',
        'quick_search_watermark': 'quickSearchWatermark',
        'resolved_quick_search_watermark': 'resolvedQuickSearchWatermark',
        'scope': 'scope',
        'scope_owner': 'scopeOwner',
        'specific_search_items': 'specificSearchItems',
        'error_code': 'errorCode',
        'error_message': 'errorMessage',
        'has_error': 'hasError'
    }

    def __init__(self, allow_quick_search=None, allow_specific_search=None, default_to_quick_search=None, display_name=None, gallery_image=None, history=None, include_available_in_specific=None, include_quick_search_in_specific=None, quick_search_id=None, quick_search_items=None, quick_search_watermark=None, resolved_quick_search_watermark=None, scope=None, scope_owner=None, specific_search_items=None, error_code=None, error_message=None, has_error=None, local_vars_configuration=None):  # noqa: E501
        """QuickSearchConfigurationResponse - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._allow_quick_search = None
        self._allow_specific_search = None
        self._default_to_quick_search = None
        self._display_name = None
        self._gallery_image = None
        self._history = None
        self._include_available_in_specific = None
        self._include_quick_search_in_specific = None
        self._quick_search_id = None
        self._quick_search_items = None
        self._quick_search_watermark = None
        self._resolved_quick_search_watermark = None
        self._scope = None
        self._scope_owner = None
        self._specific_search_items = None
        self._error_code = None
        self._error_message = None
        self._has_error = None
        self.discriminator = None

        if allow_quick_search is not None:
            self.allow_quick_search = allow_quick_search
        if allow_specific_search is not None:
            self.allow_specific_search = allow_specific_search
        if default_to_quick_search is not None:
            self.default_to_quick_search = default_to_quick_search
        if display_name is not None:
            self.display_name = display_name
        if gallery_image is not None:
            self.gallery_image = gallery_image
        if history is not None:
            self.history = history
        if include_available_in_specific is not None:
            self.include_available_in_specific = include_available_in_specific
        if include_quick_search_in_specific is not None:
            self.include_quick_search_in_specific = include_quick_search_in_specific
        if quick_search_id is not None:
            self.quick_search_id = quick_search_id
        if quick_search_items is not None:
            self.quick_search_items = quick_search_items
        if quick_search_watermark is not None:
            self.quick_search_watermark = quick_search_watermark
        if resolved_quick_search_watermark is not None:
            self.resolved_quick_search_watermark = resolved_quick_search_watermark
        if scope is not None:
            self.scope = scope
        if scope_owner is not None:
            self.scope_owner = scope_owner
        if specific_search_items is not None:
            self.specific_search_items = specific_search_items
        if error_code is not None:
            self.error_code = error_code
        if error_message is not None:
            self.error_message = error_message
        if has_error is not None:
            self.has_error = has_error

    @property
    def allow_quick_search(self):
        """Gets the allow_quick_search of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The allow_quick_search of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: bool
        """
        return self._allow_quick_search

    @allow_quick_search.setter
    def allow_quick_search(self, allow_quick_search):
        """Sets the allow_quick_search of this QuickSearchConfigurationResponse.


        :param allow_quick_search: The allow_quick_search of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: bool
        """

        self._allow_quick_search = allow_quick_search

    @property
    def allow_specific_search(self):
        """Gets the allow_specific_search of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The allow_specific_search of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: bool
        """
        return self._allow_specific_search

    @allow_specific_search.setter
    def allow_specific_search(self, allow_specific_search):
        """Sets the allow_specific_search of this QuickSearchConfigurationResponse.


        :param allow_specific_search: The allow_specific_search of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: bool
        """

        self._allow_specific_search = allow_specific_search

    @property
    def default_to_quick_search(self):
        """Gets the default_to_quick_search of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The default_to_quick_search of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: bool
        """
        return self._default_to_quick_search

    @default_to_quick_search.setter
    def default_to_quick_search(self, default_to_quick_search):
        """Sets the default_to_quick_search of this QuickSearchConfigurationResponse.


        :param default_to_quick_search: The default_to_quick_search of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: bool
        """

        self._default_to_quick_search = default_to_quick_search

    @property
    def display_name(self):
        """Gets the display_name of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The display_name of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name):
        """Sets the display_name of this QuickSearchConfigurationResponse.


        :param display_name: The display_name of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: str
        """

        self._display_name = display_name

    @property
    def gallery_image(self):
        """Gets the gallery_image of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The gallery_image of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: str
        """
        return self._gallery_image

    @gallery_image.setter
    def gallery_image(self, gallery_image):
        """Sets the gallery_image of this QuickSearchConfigurationResponse.


        :param gallery_image: The gallery_image of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: str
        """

        self._gallery_image = gallery_image

    @property
    def history(self):
        """Gets the history of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The history of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: list[str]
        """
        return self._history

    @history.setter
    def history(self, history):
        """Sets the history of this QuickSearchConfigurationResponse.


        :param history: The history of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: list[str]
        """

        self._history = history

    @property
    def include_available_in_specific(self):
        """Gets the include_available_in_specific of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The include_available_in_specific of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: bool
        """
        return self._include_available_in_specific

    @include_available_in_specific.setter
    def include_available_in_specific(self, include_available_in_specific):
        """Sets the include_available_in_specific of this QuickSearchConfigurationResponse.


        :param include_available_in_specific: The include_available_in_specific of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: bool
        """

        self._include_available_in_specific = include_available_in_specific

    @property
    def include_quick_search_in_specific(self):
        """Gets the include_quick_search_in_specific of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The include_quick_search_in_specific of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: bool
        """
        return self._include_quick_search_in_specific

    @include_quick_search_in_specific.setter
    def include_quick_search_in_specific(self, include_quick_search_in_specific):
        """Sets the include_quick_search_in_specific of this QuickSearchConfigurationResponse.


        :param include_quick_search_in_specific: The include_quick_search_in_specific of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: bool
        """

        self._include_quick_search_in_specific = include_quick_search_in_specific

    @property
    def quick_search_id(self):
        """Gets the quick_search_id of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The quick_search_id of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: str
        """
        return self._quick_search_id

    @quick_search_id.setter
    def quick_search_id(self, quick_search_id):
        """Sets the quick_search_id of this QuickSearchConfigurationResponse.


        :param quick_search_id: The quick_search_id of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: str
        """

        self._quick_search_id = quick_search_id

    @property
    def quick_search_items(self):
        """Gets the quick_search_items of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The quick_search_items of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: list[QuickSearchItem]
        """
        return self._quick_search_items

    @quick_search_items.setter
    def quick_search_items(self, quick_search_items):
        """Sets the quick_search_items of this QuickSearchConfigurationResponse.


        :param quick_search_items: The quick_search_items of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: list[QuickSearchItem]
        """

        self._quick_search_items = quick_search_items

    @property
    def quick_search_watermark(self):
        """Gets the quick_search_watermark of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The quick_search_watermark of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: str
        """
        return self._quick_search_watermark

    @quick_search_watermark.setter
    def quick_search_watermark(self, quick_search_watermark):
        """Sets the quick_search_watermark of this QuickSearchConfigurationResponse.


        :param quick_search_watermark: The quick_search_watermark of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: str
        """

        self._quick_search_watermark = quick_search_watermark

    @property
    def resolved_quick_search_watermark(self):
        """Gets the resolved_quick_search_watermark of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The resolved_quick_search_watermark of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: str
        """
        return self._resolved_quick_search_watermark

    @resolved_quick_search_watermark.setter
    def resolved_quick_search_watermark(self, resolved_quick_search_watermark):
        """Sets the resolved_quick_search_watermark of this QuickSearchConfigurationResponse.


        :param resolved_quick_search_watermark: The resolved_quick_search_watermark of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: str
        """

        self._resolved_quick_search_watermark = resolved_quick_search_watermark

    @property
    def scope(self):
        """Gets the scope of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The scope of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: str
        """
        return self._scope

    @scope.setter
    def scope(self, scope):
        """Sets the scope of this QuickSearchConfigurationResponse.


        :param scope: The scope of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: str
        """

        self._scope = scope

    @property
    def scope_owner(self):
        """Gets the scope_owner of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The scope_owner of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: str
        """
        return self._scope_owner

    @scope_owner.setter
    def scope_owner(self, scope_owner):
        """Sets the scope_owner of this QuickSearchConfigurationResponse.


        :param scope_owner: The scope_owner of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: str
        """

        self._scope_owner = scope_owner

    @property
    def specific_search_items(self):
        """Gets the specific_search_items of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The specific_search_items of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: list[QuickSearchItem]
        """
        return self._specific_search_items

    @specific_search_items.setter
    def specific_search_items(self, specific_search_items):
        """Sets the specific_search_items of this QuickSearchConfigurationResponse.


        :param specific_search_items: The specific_search_items of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: list[QuickSearchItem]
        """

        self._specific_search_items = specific_search_items

    @property
    def error_code(self):
        """Gets the error_code of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The error_code of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: str
        """
        return self._error_code

    @error_code.setter
    def error_code(self, error_code):
        """Sets the error_code of this QuickSearchConfigurationResponse.


        :param error_code: The error_code of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: str
        """

        self._error_code = error_code

    @property
    def error_message(self):
        """Gets the error_message of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The error_message of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: str
        """
        return self._error_message

    @error_message.setter
    def error_message(self, error_message):
        """Sets the error_message of this QuickSearchConfigurationResponse.


        :param error_message: The error_message of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: str
        """

        self._error_message = error_message

    @property
    def has_error(self):
        """Gets the has_error of this QuickSearchConfigurationResponse.  # noqa: E501


        :return: The has_error of this QuickSearchConfigurationResponse.  # noqa: E501
        :rtype: bool
        """
        return self._has_error

    @has_error.setter
    def has_error(self, has_error):
        """Sets the has_error of this QuickSearchConfigurationResponse.


        :param has_error: The has_error of this QuickSearchConfigurationResponse.  # noqa: E501
        :type: bool
        """

        self._has_error = has_error

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, QuickSearchConfigurationResponse):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, QuickSearchConfigurationResponse):
            return True

        return self.to_dict() != other.to_dict()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
Copyright (C) 2019 Paul Greenberg @greenpau
See LICENSE for licensing details
'''

from __future__ import (absolute_import, division, print_function)
import os
import sys
import stat
import logging
try:
    from configparser import RawConfigParser
except ImportError:
    from ConfigParser import RawConfigParser  # ver. < 3.0
import base64
import re
from pprint import pprint
import time
import json

__pkg_name__ = 'pycherwell'
__author__ = 'Paul Greenberg @greenpau'
__version__ = '1.0.9'
__maintainer__ = 'Paul Greenberg'
__email__ = 'greenpau@outlook.com'
__status__ = 'Alpha'

class AppConfiguration(object):
    '''
    This class implements the client application configuration.
    '''

    def __init__(self, config_file, profile):
        ''' Initializes the class. '''
        self.app_name = __pkg_name__
        self.app_version = __version__
        self.config_file = config_file
        if not self.config_file:
            self.app_conf_dir = os.path.expanduser('~/.%s' % (self.app_name.lstrip('py')))
            self.config_file = os.path.join(self.app_conf_dir, 'config')
        else:
            self.app_conf_dir = os.path.dirname(config_file)
        self.profile = profile
        self.settings = {}
        self.log = logging.getLogger('%s-config' % (self.app_name))
        handler = logging.StreamHandler(sys.stderr)
        formatter = logging.Formatter('[%(asctime)s] %(levelname)s %(name)s@%(lineno)d: %(message)s')
        handler.setFormatter(formatter)
        self.log.addHandler(handler)
        self.debug_enabled = False
        self.cfg_keys = [
            'client_id',
            'auth_mode',
            'username',
            'password',
            'host',
            'port',
            'protocol',
            'basepath'
        ]
        self.env_key_pairs = []
        for k in self.cfg_keys:
            self.env_key_pairs.append(('%s_%s' % (self.app_name, k.upper()), k))
        self.verify_ssl = False
        self.ssl_ca_cert = False
        self.assert_hostname = None
        self.connection_pool_maxsize = 4
        self.proxy = None
        self.cert_file = None
        self.key_file = None
        self.safe_chars_for_path_param = ''
        self.app_conf_file_names = {
            'token': os.path.join(self.app_conf_dir, 'token.json'),
            'business_objects': os.path.join(self.app_conf_dir, 'business_objects.json'),
            'teams': os.path.join(self.app_conf_dir, 'teams'),
        }
        self.app = {}
        for k in self.app_conf_file_names:
            self.app[k] = None
        return


    def load(self):
        cfg_file = self.config_file
        ''' Load configuration from a configuration file in RC format. '''
        cfg_file = os.path.expanduser(cfg_file)
        self.log.debug('configuration file: %s', cfg_file)

        _has_cfg_file = True
        if not os.path.exists(cfg_file):
            self.log.debug('configuration file: %s does not exist ', cfg_file)
            _has_cfg_file = False

        ''' Validate configuration file permissions '''
        if _has_cfg_file:
            cfg_file_stat = os.stat(cfg_file)
            if cfg_file_stat.st_mode & stat.S_IROTH:
                raise Exception('config', 'configuration file %s is world readable' % cfg_file)
            if cfg_file_stat.st_mode & stat.S_IRGRP:
                raise Exception('config', 'configuration file %s is group readable' % cfg_file)

        ''' Handle configuration file '''
        if _has_cfg_file:
            self.cfg_file = cfg_file
            cfg_parser = RawConfigParser()
            cfg_parser.read(cfg_file)
            if self.profile not in cfg_parser.sections():
                raise Exception('config', 'configuration file %s has no %s section' % (cfg_file, self.profile))
            for cfg_key in self.cfg_keys:
                if cfg_parser.has_option(self.profile, cfg_key):
                    self.settings[cfg_key] = cfg_parser.get(self.profile, cfg_key)
                    cfg_value = self.settings[cfg_key]
                    if re.match('password', cfg_key):
                        cfg_value = "*************"
                    self.log.debug('config file: key = %s, value = %s', cfg_key, cfg_value)

        ''' Handle environment variables '''
        for env_key_pair in self.env_key_pairs:
            if not os.environ.get(env_key_pair[0]):
                continue
            cfg_key = env_key_pair[1]
            cfg_value = os.environ.get(env_key_pair[0])
            self.settings[cfg_key] = cfg_value
            if re.match('password', cfg_key):
                cfg_value = "*************"
            self.log.debug('environment var: key = %s, value = %s', cfg_key, cfg_value)

        self.validate()
        return

    def validate(self):
        '''
        Validates that all configuration parameters necessary to establish
        a connection to SEP Manager are present.
        '''
        for cfg_key in self.cfg_keys:
            if cfg_key not in self.settings:
                if cfg_key == 'protocol':
                    self.settings[cfg_key] = 'https'
                elif cfg_key == 'port':
                    self.settings[cfg_key] = '443'
                elif cfg_key == 'basepath':
                    self.settings[cfg_key] = '/CherwellAPI'
                elif cfg_key == 'host':
                    self.settings[cfg_key] = 'localhost'
                else:
                    raise Exception('config', \
                            "no '%s' key in '%s' " % (cfg_key, self.profile) + \
                            "section of the configuration")
            else:
                self.settings[cfg_key] = self.settings[cfg_key].strip("'").strip('"')
        return

    def get_host(self):
        if self.settings['protocol'] == 'https' and self.settings['port'] == '443':
            return '%s://%s/%s' % (self.settings['protocol'], self.settings['host'], self.settings['basepath'])
        else:
            return '%s://%s:%s/%s' % (self.settings['protocol'], self.settings['host'], self.settings['port'], self.settings['basepath'])

    def get_auth_url(self):
        ''' Return base path URL. '''
        base = self.get_host()
        base += '/token?'
        base += 'auth_mode=' + self.settings['auth_mode']
        base += '&api_key=' + self.settings['client_id']
        return base

    def get_auth_req_fields(self):
        fields = {}
        fields['grant_type'] = 'password'
        if self.settings['auth_mode'].lower() in ['internal', 'ldap']:
            fields['client_id'] = self.settings['client_id']
            fields['username'] = self.settings['username']
            fields['password'] = self.settings['password']
        return fields

    def get_verify_ssl(self):
        return self.verify_ssl

    def get_ssl_ca_cert(self):
        return self.ssl_ca_cert

    def get_assert_hostname(self):
        return self.assert_hostname

    def get_connection_pool_maxsize(self):
        return self.connection_pool_maxsize

    def get_proxy(self):
        return self.proxy

    def get_cert_file(self):
        return self.cert_file

    def get_key_file(self):
        return self.key_file

    def get_safe_chars_for_path_param(self):
        return self.safe_chars_for_path_param

    def set_verify_ssl(self, param=False):
        self.verify_ssl = param
        return

    def set_ssl_ca_cert(self, param=False):
        self.ssl_ca_cert = param
        return

    def set_assert_hostname(self, param=None):
        self.assert_hostname = param
        return

    def set_connection_pool_maxsize(self, param=4):
        self.connection_pool_maxsize = param
        return

    def set_proxy(self, param=None):
        self.proxy = param
        return

    def set_cert_file(self, param=None):
        self.cert_file = param
        return

    def set_key_file(self, param=None):
        self.key_file = param
        return

    def set_safe_chars_for_path_param(self, param=''):
        self.safe_chars_for_path_param = param
        return

    def get_auth_header_name(self):
        return 'Authorization'

    def get_auth_header_value(self):
        return 'Bearer %s' % (self.app['token']['access_token'])

    def get_user_agent(self):
        return '%s/%s' % (self.app_name, self.app_version)

    def get_auth_token(self):
        self.load_app_section('token')
        self.log.debug('Authentication token: %s', self.app['token'])
        if self.app['token'] is not None:
            if 'expires_at_timestamp' not in self.app['token']:
                self.app['token'] = None
                self.wipe_app_section('token')
                return self.app['token']
            if int(time.time()) > self.app['token']['expires_at_timestamp']:
                self.app['token'] = None
                self.wipe_app_section('token')
                return self.app['token']
        return self.app['token']


    def enrich_app_section(self, section):
        if section not in ['business_objects']:
            return
        ref_section = section + '_ref'
        if ref_section not in self.app:
            self.app[ref_section] = {}
        mandatory_fields = ['bus_ob_id', 'name', 'major']
        for entry in self.app[section]:
            is_valid_entry = True
            for f in mandatory_fields:
                if f not in entry:
                    is_valid_entry = False
                    break
            if not is_valid_entry:
                continue
            #if entry['major'] != True:
            #    continue
            self.app[ref_section][entry['name']] = entry
        return

    def get_business_object_id(self, name):
        if name in self.app['business_objects_ref']:
            return self.app['business_objects_ref'][name]['bus_ob_id']
        return None

    def get_business_object_field_id(self, busObId, busObFieldName):
        if 'business_object_fields_ref' not in self.app:
            raise Exception('client', 'business_object_fields_ref not found')
        if busObId not in self.app['business_object_fields_ref']:
            raise Exception('client', 'BO %s not found in business_object_fields_ref' % (busObId))
        for entry in self.app['business_object_fields_ref'][busObId]:
            if 'field_id' not in entry or 'display_name' not in entry:
                continue
            for k in entry:
                if k not in ['display_name', 'name']:
                    continue
                if entry[k] == busObFieldName:
                    return entry['field_id']
        raise Exception('client', 'field %s not found in BO %s fields in business_object_fields_ref' % (busObFieldName, busObId))

    def load_app_data(self, partition, key):
        fn = '%s_%s.json' % (partition, key)
        fp = os.path.join(self.app_conf_dir, fn)
        self.log.debug('loading app data for %s/%s from %s', partition, key, fp)
        if not os.path.exists(fp):
            return False
        if os.stat(fp).st_size < 100:
            return False
        data = None
        with open(fp, 'r') as fh:
            data = json.load(fh)
        if not data:
            self.log.error('failed loading app data for %s/%s from %s', partition, key, fp)
            return False
        self.log.debug('loaded app data for %s/%s from %s', partition, key, fp)
        if partition not in self.app:
            self.app[partition] = {}
        self.app[partition][key] = data
        return True

    def save_app_data(self, partition, key, data):
        fn = '%s_%s.json' % (partition, key)
        fp = os.path.join(self.app_conf_dir, fn)
        self.log.debug('saving app data for %s/%s in %s', partition, key, fp)
        try:
            if not os.path.exists(self.app_conf_dir):
                os.makedirs(self.app_conf_dir)
        except:
            pass
        with open(fp, 'w') as f:
            formatted_data = json.dumps(data, sort_keys=True, indent=4, separators=(',', ': '))
            f.write(formatted_data)
            f.write('\n')
        if partition not in self.app:
            self.app[partition] = {}
        self.app[partition][key] = data

    def load_app_section(self, section):
        if section not in self.app_conf_file_names:
            self.log.error('The app config %s section is unsupported', section)
            return False
        if section in self.app:
            if self.app[section] is not None:
                return True
        fn = self.app_conf_file_names[section]
        if os.path.exists(fn):
            if os.stat(fn).st_size > 100:
                with open(fn, 'r') as f:
                    self.app[section] = json.load(f)
                self.log.debug('Loaded app config %s section from %s', section, fn)
                self.enrich_app_section(section)
        return True

    def save_app_section(self, section, data):
        if section not in self.app_conf_file_names:
            self.log.error('The app config %s section is unsupported', section)
            return
        fn = self.app_conf_file_names[section]
        self.log.debug('Saving app config %s section from %s', section, fn)
        try:
            if not os.path.exists(self.app_conf_dir):
                os.makedirs(self.app_conf_dir)
        except:
            pass
        if section == 'token':
            data['expires_at_timestamp'] = int(time.time()) + data['expires_in']
        with open(fn, 'w') as f:
            formatted_data = json.dumps(data, sort_keys=True, indent=4, separators=(',', ': '))
            f.write(formatted_data)
            f.write('\n')
        self.app[section] = data
        if section + '_ref' not in self.app:
            self.enrich_app_section(section)
        return

    def wipe_app_section(self, section):
        if section not in self.app_conf_file_names:
            self.log.error('The app config %s section is unsupported', section)
            return
        fn = self.app_conf_file_names[section]
        with open(fn, 'w') as f:
            pass
        self.app[section] = None
        return



# coding: utf-8

"""
    Cherwell REST API

    Unofficial Python Cherwell REST API library.  # noqa: E501

    The version of the OpenAPI document: 9.3.2
    Contact: See AUTHORS.
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from pycherwell.configuration import Configuration


class NameValuePair(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'value_object': 'object',
        'value_string': 'str',
        'category': 'str',
        'description': 'str',
        'display_shows_value': 'bool',
        'special_use_flag': 'bool',
        'display_string': 'str'
    }

    attribute_map = {
        'name': 'name',
        'value_object': 'valueObject',
        'value_string': 'valueString',
        'category': 'category',
        'description': 'description',
        'display_shows_value': 'displayShowsValue',
        'special_use_flag': 'specialUseFlag',
        'display_string': 'displayString'
    }

    def __init__(self, name=None, value_object=None, value_string=None, category=None, description=None, display_shows_value=None, special_use_flag=None, display_string=None, local_vars_configuration=None):  # noqa: E501
        """NameValuePair - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._name = None
        self._value_object = None
        self._value_string = None
        self._category = None
        self._description = None
        self._display_shows_value = None
        self._special_use_flag = None
        self._display_string = None
        self.discriminator = None

        if name is not None:
            self.name = name
        if value_object is not None:
            self.value_object = value_object
        if value_string is not None:
            self.value_string = value_string
        if category is not None:
            self.category = category
        if description is not None:
            self.description = description
        if display_shows_value is not None:
            self.display_shows_value = display_shows_value
        if special_use_flag is not None:
            self.special_use_flag = special_use_flag
        if display_string is not None:
            self.display_string = display_string

    @property
    def name(self):
        """Gets the name of this NameValuePair.  # noqa: E501


        :return: The name of this NameValuePair.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this NameValuePair.


        :param name: The name of this NameValuePair.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def value_object(self):
        """Gets the value_object of this NameValuePair.  # noqa: E501


        :return: The value_object of this NameValuePair.  # noqa: E501
        :rtype: object
        """
        return self._value_object

    @value_object.setter
    def value_object(self, value_object):
        """Sets the value_object of this NameValuePair.


        :param value_object: The value_object of this NameValuePair.  # noqa: E501
        :type: object
        """

        self._value_object = value_object

    @property
    def value_string(self):
        """Gets the value_string of this NameValuePair.  # noqa: E501


        :return: The value_string of this NameValuePair.  # noqa: E501
        :rtype: str
        """
        return self._value_string

    @value_string.setter
    def value_string(self, value_string):
        """Sets the value_string of this NameValuePair.


        :param value_string: The value_string of this NameValuePair.  # noqa: E501
        :type: str
        """

        self._value_string = value_string

    @property
    def category(self):
        """Gets the category of this NameValuePair.  # noqa: E501


        :return: The category of this NameValuePair.  # noqa: E501
        :rtype: str
        """
        return self._category

    @category.setter
    def category(self, category):
        """Sets the category of this NameValuePair.


        :param category: The category of this NameValuePair.  # noqa: E501
        :type: str
        """

        self._category = category

    @property
    def description(self):
        """Gets the description of this NameValuePair.  # noqa: E501


        :return: The description of this NameValuePair.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this NameValuePair.


        :param description: The description of this NameValuePair.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def display_shows_value(self):
        """Gets the display_shows_value of this NameValuePair.  # noqa: E501


        :return: The display_shows_value of this NameValuePair.  # noqa: E501
        :rtype: bool
        """
        return self._display_shows_value

    @display_shows_value.setter
    def display_shows_value(self, display_shows_value):
        """Sets the display_shows_value of this NameValuePair.


        :param display_shows_value: The display_shows_value of this NameValuePair.  # noqa: E501
        :type: bool
        """

        self._display_shows_value = display_shows_value

    @property
    def special_use_flag(self):
        """Gets the special_use_flag of this NameValuePair.  # noqa: E501


        :return: The special_use_flag of this NameValuePair.  # noqa: E501
        :rtype: bool
        """
        return self._special_use_flag

    @special_use_flag.setter
    def special_use_flag(self, special_use_flag):
        """Sets the special_use_flag of this NameValuePair.


        :param special_use_flag: The special_use_flag of this NameValuePair.  # noqa: E501
        :type: bool
        """

        self._special_use_flag = special_use_flag

    @property
    def display_string(self):
        """Gets the display_string of this NameValuePair.  # noqa: E501


        :return: The display_string of this NameValuePair.  # noqa: E501
        :rtype: str
        """
        return self._display_string

    @display_string.setter
    def display_string(self, display_string):
        """Sets the display_string of this NameValuePair.


        :param display_string: The display_string of this NameValuePair.  # noqa: E501
        :type: str
        """

        self._display_string = display_string

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, NameValuePair):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, NameValuePair):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Cherwell REST API

    Unofficial Python Cherwell REST API library.  # noqa: E501

    The version of the OpenAPI document: 9.3.2
    Contact: See AUTHORS.
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest

import pycherwell
from pycherwell.api.teams_api import TeamsApi  # noqa: E501
from pycherwell.rest import ApiException


class TestTeamsApi(unittest.TestCase):
    """TeamsApi unit test stubs"""

    def setUp(self):
        self.api = pycherwell.api.teams_api.TeamsApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_teams_add_user_to_team_by_batch_v1(self):
        """Test case for teams_add_user_to_team_by_batch_v1

        Add users to a team by batch  # noqa: E501
        """
        pass

    def test_teams_add_user_to_team_v1(self):
        """Test case for teams_add_user_to_team_v1

        Add a user to a team  # noqa: E501
        """
        pass

    def test_teams_add_user_to_team_v2(self):
        """Test case for teams_add_user_to_team_v2

        Add a user to a team  # noqa: E501
        """
        pass

    def test_teams_delete_team_v1(self):
        """Test case for teams_delete_team_v1

        Delete a Team  # noqa: E501
        """
        pass

    def test_teams_get_team_v1(self):
        """Test case for teams_get_team_v1

        Get a team by its TeamId  # noqa: E501
        """
        pass

    def test_teams_get_teams_v1(self):
        """Test case for teams_get_teams_v1

        Get all available Teams  # noqa: E501
        """
        pass

    def test_teams_get_teams_v2(self):
        """Test case for teams_get_teams_v2

        Get all available Teams  # noqa: E501
        """
        pass

    def test_teams_get_users_teams_v1(self):
        """Test case for teams_get_users_teams_v1

        Get Team assignments for a user  # noqa: E501
        """
        pass

    def test_teams_get_users_teams_v2(self):
        """Test case for teams_get_users_teams_v2

        Get Team assignments for a user  # noqa: E501
        """
        pass

    def test_teams_get_workgroups_v1(self):
        """Test case for teams_get_workgroups_v1

        Get all available Workgroups  # noqa: E501
        """
        pass

    def test_teams_get_workgroups_v2(self):
        """Test case for teams_get_workgroups_v2

        Get all available Workgroups  # noqa: E501
        """
        pass

    def test_teams_remove_customer_from_workgroup_v1(self):
        """Test case for teams_remove_customer_from_workgroup_v1

        Remove a customer from a Workgroup  # noqa: E501
        """
        pass

    def test_teams_remove_user_from_team_v1(self):
        """Test case for teams_remove_user_from_team_v1

        Operation to remove a User from a Team.  # noqa: E501
        """
        pass

    def test_teams_remove_user_from_team_v2(self):
        """Test case for teams_remove_user_from_team_v2

        Operation to remove a User from a Team.  # noqa: E501
        """
        pass

    def test_teams_save_team_member_v1(self):
        """Test case for teams_save_team_member_v1

        Add or Update a team member  # noqa: E501
        """
        pass

    def test_teams_save_team_v1(self):
        """Test case for teams_save_team_v1

        Create or update a team  # noqa: E501
        """
        pass

    def test_teams_save_workgroup_member_v1(self):
        """Test case for teams_save_workgroup_member_v1

        Save the membership status of a Workgroup member.  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Cherwell REST API

    Unofficial Python Cherwell REST API library.  # noqa: E501

    The version of the OpenAPI document: 9.3.2
    Contact: See AUTHORS.
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from pycherwell.configuration import Configuration


class SearchItem(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'association': 'str',
        'links': 'list[Link]',
        'localized_scope_name': 'str',
        'parent_folder_id': 'str',
        'scope': 'str',
        'scope_owner': 'str',
        'search_id': 'str',
        'search_name': 'str'
    }

    attribute_map = {
        'association': 'association',
        'links': 'links',
        'localized_scope_name': 'localizedScopeName',
        'parent_folder_id': 'parentFolderId',
        'scope': 'scope',
        'scope_owner': 'scopeOwner',
        'search_id': 'searchId',
        'search_name': 'searchName'
    }

    def __init__(self, association=None, links=None, localized_scope_name=None, parent_folder_id=None, scope=None, scope_owner=None, search_id=None, search_name=None, local_vars_configuration=None):  # noqa: E501
        """SearchItem - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._association = None
        self._links = None
        self._localized_scope_name = None
        self._parent_folder_id = None
        self._scope = None
        self._scope_owner = None
        self._search_id = None
        self._search_name = None
        self.discriminator = None

        if association is not None:
            self.association = association
        if links is not None:
            self.links = links
        if localized_scope_name is not None:
            self.localized_scope_name = localized_scope_name
        if parent_folder_id is not None:
            self.parent_folder_id = parent_folder_id
        if scope is not None:
            self.scope = scope
        if scope_owner is not None:
            self.scope_owner = scope_owner
        if search_id is not None:
            self.search_id = search_id
        if search_name is not None:
            self.search_name = search_name

    @property
    def association(self):
        """Gets the association of this SearchItem.  # noqa: E501


        :return: The association of this SearchItem.  # noqa: E501
        :rtype: str
        """
        return self._association

    @association.setter
    def association(self, association):
        """Sets the association of this SearchItem.


        :param association: The association of this SearchItem.  # noqa: E501
        :type: str
        """

        self._association = association

    @property
    def links(self):
        """Gets the links of this SearchItem.  # noqa: E501


        :return: The links of this SearchItem.  # noqa: E501
        :rtype: list[Link]
        """
        return self._links

    @links.setter
    def links(self, links):
        """Sets the links of this SearchItem.


        :param links: The links of this SearchItem.  # noqa: E501
        :type: list[Link]
        """

        self._links = links

    @property
    def localized_scope_name(self):
        """Gets the localized_scope_name of this SearchItem.  # noqa: E501


        :return: The localized_scope_name of this SearchItem.  # noqa: E501
        :rtype: str
        """
        return self._localized_scope_name

    @localized_scope_name.setter
    def localized_scope_name(self, localized_scope_name):
        """Sets the localized_scope_name of this SearchItem.


        :param localized_scope_name: The localized_scope_name of this SearchItem.  # noqa: E501
        :type: str
        """

        self._localized_scope_name = localized_scope_name

    @property
    def parent_folder_id(self):
        """Gets the parent_folder_id of this SearchItem.  # noqa: E501


        :return: The parent_folder_id of this SearchItem.  # noqa: E501
        :rtype: str
        """
        return self._parent_folder_id

    @parent_folder_id.setter
    def parent_folder_id(self, parent_folder_id):
        """Sets the parent_folder_id of this SearchItem.


        :param parent_folder_id: The parent_folder_id of this SearchItem.  # noqa: E501
        :type: str
        """

        self._parent_folder_id = parent_folder_id

    @property
    def scope(self):
        """Gets the scope of this SearchItem.  # noqa: E501


        :return: The scope of this SearchItem.  # noqa: E501
        :rtype: str
        """
        return self._scope

    @scope.setter
    def scope(self, scope):
        """Sets the scope of this SearchItem.


        :param scope: The scope of this SearchItem.  # noqa: E501
        :type: str
        """

        self._scope = scope

    @property
    def scope_owner(self):
        """Gets the scope_owner of this SearchItem.  # noqa: E501


        :return: The scope_owner of this SearchItem.  # noqa: E501
        :rtype: str
        """
        return self._scope_owner

    @scope_owner.setter
    def scope_owner(self, scope_owner):
        """Sets the scope_owner of this SearchItem.


        :param scope_owner: The scope_owner of this SearchItem.  # noqa: E501
        :type: str
        """

        self._scope_owner = scope_owner

    @property
    def search_id(self):
        """Gets the search_id of this SearchItem.  # noqa: E501


        :return: The search_id of this SearchItem.  # noqa: E501
        :rtype: str
        """
        return self._search_id

    @search_id.setter
    def search_id(self, search_id):
        """Sets the search_id of this SearchItem.


        :param search_id: The search_id of this SearchItem.  # noqa: E501
        :type: str
        """

        self._search_id = search_id

    @property
    def search_name(self):
        """Gets the search_name of this SearchItem.  # noqa: E501


        :return: The search_name of this SearchItem.  # noqa: E501
        :rtype: str
        """
        return self._search_name

    @search_name.setter
    def search_name(self, search_name):
        """Sets the search_name of this SearchItem.


        :param search_name: The search_name of this SearchItem.  # noqa: E501
        :type: str
        """

        self._search_name = search_name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SearchItem):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SearchItem):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Cherwell REST API

    Unofficial Python Cherwell REST API library.  # noqa: E501

    The version of the OpenAPI document: 9.3.2
    Contact: See AUTHORS.
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest

import pycherwell
from pycherwell.api.core_api import CoreApi  # noqa: E501
from pycherwell.rest import ApiException


class TestCoreApi(unittest.TestCase):
    """CoreApi unit test stubs"""

    def setUp(self):
        self.api = pycherwell.api.core_api.CoreApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_core_delete_gallery_image_by_stand_in_key_v1(self):
        """Test case for core_delete_gallery_image_by_stand_in_key_v1

        Delete a gallery image  # noqa: E501
        """
        pass

    def test_core_get_gallery_image_v1(self):
        """Test case for core_get_gallery_image_v1

        Get built-in images  # noqa: E501
        """
        pass

    def test_core_get_gallery_images_folder_v1(self):
        """Test case for core_get_gallery_images_folder_v1

        Get gallery images by scope, scopeowner, and folder  # noqa: E501
        """
        pass

    def test_core_get_gallery_images_scope_owner_v1(self):
        """Test case for core_get_gallery_images_scope_owner_v1

        Get gallery images by scope and scopeowner  # noqa: E501
        """
        pass

    def test_core_get_gallery_images_scope_v1(self):
        """Test case for core_get_gallery_images_scope_v1

        Get gallery images by scope  # noqa: E501
        """
        pass

    def test_core_get_gallery_images_v1(self):
        """Test case for core_get_gallery_images_v1

        Get all gallery images  # noqa: E501
        """
        pass

    def test_core_get_stored_value_v1(self):
        """Test case for core_get_stored_value_v1

        Get a  stored value  # noqa: E501
        """
        pass

    def test_core_get_stored_values_folder_v1(self):
        """Test case for core_get_stored_values_folder_v1

        Get stored values by folder  # noqa: E501
        """
        pass

    def test_core_get_stored_values_scope_owner_v1(self):
        """Test case for core_get_stored_values_scope_owner_v1

        Get stored values by scope owner  # noqa: E501
        """
        pass

    def test_core_get_stored_values_scope_v1(self):
        """Test case for core_get_stored_values_scope_v1

        Get stored values by scope  # noqa: E501
        """
        pass

    def test_core_get_stored_values_v1(self):
        """Test case for core_get_stored_values_v1

        Gets all the stored values in the system  # noqa: E501
        """
        pass

    def test_core_get_views_v1(self):
        """Test case for core_get_views_v1

        Get a list of the views  # noqa: E501
        """
        pass

    def test_core_save_gallery_image_v1(self):
        """Test case for core_save_gallery_image_v1

        Create or update a gallery image  # noqa: E501
        """
        pass

    def test_core_save_stored_value_v1(self):
        """Test case for core_save_stored_value_v1

        Create or update a stored value  # noqa: E501
        """
        pass

    def test_core_set_culture_v1(self):
        """Test case for core_set_culture_v1

        Set the culture for the current user  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Cherwell REST API

    Unofficial Python Cherwell REST API library.  # noqa: E501

    The version of the OpenAPI document: 9.3.2
    Contact: See AUTHORS.
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from pycherwell.configuration import Configuration


class RelatedBusinessObjectRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'all_fields': 'bool',
        'custom_grid_id': 'str',
        'fields_list': 'list[str]',
        'filters': 'list[FilterInfo]',
        'page_number': 'int',
        'page_size': 'int',
        'parent_bus_ob_id': 'str',
        'parent_bus_ob_rec_id': 'str',
        'relationship_id': 'str',
        'sorting': 'list[SortInfo]',
        'use_default_grid': 'bool'
    }

    attribute_map = {
        'all_fields': 'allFields',
        'custom_grid_id': 'customGridId',
        'fields_list': 'fieldsList',
        'filters': 'filters',
        'page_number': 'pageNumber',
        'page_size': 'pageSize',
        'parent_bus_ob_id': 'parentBusObId',
        'parent_bus_ob_rec_id': 'parentBusObRecId',
        'relationship_id': 'relationshipId',
        'sorting': 'sorting',
        'use_default_grid': 'useDefaultGrid'
    }

    def __init__(self, all_fields=None, custom_grid_id=None, fields_list=None, filters=None, page_number=None, page_size=None, parent_bus_ob_id=None, parent_bus_ob_rec_id=None, relationship_id=None, sorting=None, use_default_grid=None, local_vars_configuration=None):  # noqa: E501
        """RelatedBusinessObjectRequest - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._all_fields = None
        self._custom_grid_id = None
        self._fields_list = None
        self._filters = None
        self._page_number = None
        self._page_size = None
        self._parent_bus_ob_id = None
        self._parent_bus_ob_rec_id = None
        self._relationship_id = None
        self._sorting = None
        self._use_default_grid = None
        self.discriminator = None

        if all_fields is not None:
            self.all_fields = all_fields
        if custom_grid_id is not None:
            self.custom_grid_id = custom_grid_id
        if fields_list is not None:
            self.fields_list = fields_list
        if filters is not None:
            self.filters = filters
        if page_number is not None:
            self.page_number = page_number
        if page_size is not None:
            self.page_size = page_size
        if parent_bus_ob_id is not None:
            self.parent_bus_ob_id = parent_bus_ob_id
        if parent_bus_ob_rec_id is not None:
            self.parent_bus_ob_rec_id = parent_bus_ob_rec_id
        if relationship_id is not None:
            self.relationship_id = relationship_id
        if sorting is not None:
            self.sorting = sorting
        if use_default_grid is not None:
            self.use_default_grid = use_default_grid

    @property
    def all_fields(self):
        """Gets the all_fields of this RelatedBusinessObjectRequest.  # noqa: E501


        :return: The all_fields of this RelatedBusinessObjectRequest.  # noqa: E501
        :rtype: bool
        """
        return self._all_fields

    @all_fields.setter
    def all_fields(self, all_fields):
        """Sets the all_fields of this RelatedBusinessObjectRequest.


        :param all_fields: The all_fields of this RelatedBusinessObjectRequest.  # noqa: E501
        :type: bool
        """

        self._all_fields = all_fields

    @property
    def custom_grid_id(self):
        """Gets the custom_grid_id of this RelatedBusinessObjectRequest.  # noqa: E501


        :return: The custom_grid_id of this RelatedBusinessObjectRequest.  # noqa: E501
        :rtype: str
        """
        return self._custom_grid_id

    @custom_grid_id.setter
    def custom_grid_id(self, custom_grid_id):
        """Sets the custom_grid_id of this RelatedBusinessObjectRequest.


        :param custom_grid_id: The custom_grid_id of this RelatedBusinessObjectRequest.  # noqa: E501
        :type: str
        """

        self._custom_grid_id = custom_grid_id

    @property
    def fields_list(self):
        """Gets the fields_list of this RelatedBusinessObjectRequest.  # noqa: E501


        :return: The fields_list of this RelatedBusinessObjectRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._fields_list

    @fields_list.setter
    def fields_list(self, fields_list):
        """Sets the fields_list of this RelatedBusinessObjectRequest.


        :param fields_list: The fields_list of this RelatedBusinessObjectRequest.  # noqa: E501
        :type: list[str]
        """

        self._fields_list = fields_list

    @property
    def filters(self):
        """Gets the filters of this RelatedBusinessObjectRequest.  # noqa: E501


        :return: The filters of this RelatedBusinessObjectRequest.  # noqa: E501
        :rtype: list[FilterInfo]
        """
        return self._filters

    @filters.setter
    def filters(self, filters):
        """Sets the filters of this RelatedBusinessObjectRequest.


        :param filters: The filters of this RelatedBusinessObjectRequest.  # noqa: E501
        :type: list[FilterInfo]
        """

        self._filters = filters

    @property
    def page_number(self):
        """Gets the page_number of this RelatedBusinessObjectRequest.  # noqa: E501


        :return: The page_number of this RelatedBusinessObjectRequest.  # noqa: E501
        :rtype: int
        """
        return self._page_number

    @page_number.setter
    def page_number(self, page_number):
        """Sets the page_number of this RelatedBusinessObjectRequest.


        :param page_number: The page_number of this RelatedBusinessObjectRequest.  # noqa: E501
        :type: int
        """

        self._page_number = page_number

    @property
    def page_size(self):
        """Gets the page_size of this RelatedBusinessObjectRequest.  # noqa: E501


        :return: The page_size of this RelatedBusinessObjectRequest.  # noqa: E501
        :rtype: int
        """
        return self._page_size

    @page_size.setter
    def page_size(self, page_size):
        """Sets the page_size of this RelatedBusinessObjectRequest.


        :param page_size: The page_size of this RelatedBusinessObjectRequest.  # noqa: E501
        :type: int
        """

        self._page_size = page_size

    @property
    def parent_bus_ob_id(self):
        """Gets the parent_bus_ob_id of this RelatedBusinessObjectRequest.  # noqa: E501


        :return: The parent_bus_ob_id of this RelatedBusinessObjectRequest.  # noqa: E501
        :rtype: str
        """
        return self._parent_bus_ob_id

    @parent_bus_ob_id.setter
    def parent_bus_ob_id(self, parent_bus_ob_id):
        """Sets the parent_bus_ob_id of this RelatedBusinessObjectRequest.


        :param parent_bus_ob_id: The parent_bus_ob_id of this RelatedBusinessObjectRequest.  # noqa: E501
        :type: str
        """

        self._parent_bus_ob_id = parent_bus_ob_id

    @property
    def parent_bus_ob_rec_id(self):
        """Gets the parent_bus_ob_rec_id of this RelatedBusinessObjectRequest.  # noqa: E501


        :return: The parent_bus_ob_rec_id of this RelatedBusinessObjectRequest.  # noqa: E501
        :rtype: str
        """
        return self._parent_bus_ob_rec_id

    @parent_bus_ob_rec_id.setter
    def parent_bus_ob_rec_id(self, parent_bus_ob_rec_id):
        """Sets the parent_bus_ob_rec_id of this RelatedBusinessObjectRequest.


        :param parent_bus_ob_rec_id: The parent_bus_ob_rec_id of this RelatedBusinessObjectRequest.  # noqa: E501
        :type: str
        """

        self._parent_bus_ob_rec_id = parent_bus_ob_rec_id

    @property
    def relationship_id(self):
        """Gets the relationship_id of this RelatedBusinessObjectRequest.  # noqa: E501


        :return: The relationship_id of this RelatedBusinessObjectRequest.  # noqa: E501
        :rtype: str
        """
        return self._relationship_id

    @relationship_id.setter
    def relationship_id(self, relationship_id):
        """Sets the relationship_id of this RelatedBusinessObjectRequest.


        :param relationship_id: The relationship_id of this RelatedBusinessObjectRequest.  # noqa: E501
        :type: str
        """

        self._relationship_id = relationship_id

    @property
    def sorting(self):
        """Gets the sorting of this RelatedBusinessObjectRequest.  # noqa: E501


        :return: The sorting of this RelatedBusinessObjectRequest.  # noqa: E501
        :rtype: list[SortInfo]
        """
        return self._sorting

    @sorting.setter
    def sorting(self, sorting):
        """Sets the sorting of this RelatedBusinessObjectRequest.


        :param sorting: The sorting of this RelatedBusinessObjectRequest.  # noqa: E501
        :type: list[SortInfo]
        """

        self._sorting = sorting

    @property
    def use_default_grid(self):
        """Gets the use_default_grid of this RelatedBusinessObjectRequest.  # noqa: E501


        :return: The use_default_grid of this RelatedBusinessObjectRequest.  # noqa: E501
        :rtype: bool
        """
        return self._use_default_grid

    @use_default_grid.setter
    def use_default_grid(self, use_default_grid):
        """Sets the use_default_grid of this RelatedBusinessObjectRequest.


        :param use_default_grid: The use_default_grid of this RelatedBusinessObjectRequest.  # noqa: E501
        :type: bool
        """

        self._use_default_grid = use_default_grid

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RelatedBusinessObjectRequest):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, RelatedBusinessObjectRequest):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Cherwell REST API

    Unofficial Python Cherwell REST API library.  # noqa: E501

    The version of the OpenAPI document: 9.3.2
    Contact: See AUTHORS.
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from pycherwell.configuration import Configuration


class AttachmentsRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'attachment_id': 'str',
        'attachment_types': 'list[str]',
        'bus_ob_id': 'str',
        'bus_ob_name': 'str',
        'bus_ob_public_id': 'str',
        'bus_ob_rec_id': 'str',
        'include_links': 'bool',
        'types': 'list[str]'
    }

    attribute_map = {
        'attachment_id': 'attachmentId',
        'attachment_types': 'attachmentTypes',
        'bus_ob_id': 'busObId',
        'bus_ob_name': 'busObName',
        'bus_ob_public_id': 'busObPublicId',
        'bus_ob_rec_id': 'busObRecId',
        'include_links': 'includeLinks',
        'types': 'types'
    }

    def __init__(self, attachment_id=None, attachment_types=None, bus_ob_id=None, bus_ob_name=None, bus_ob_public_id=None, bus_ob_rec_id=None, include_links=None, types=None, local_vars_configuration=None):  # noqa: E501
        """AttachmentsRequest - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._attachment_id = None
        self._attachment_types = None
        self._bus_ob_id = None
        self._bus_ob_name = None
        self._bus_ob_public_id = None
        self._bus_ob_rec_id = None
        self._include_links = None
        self._types = None
        self.discriminator = None

        if attachment_id is not None:
            self.attachment_id = attachment_id
        if attachment_types is not None:
            self.attachment_types = attachment_types
        if bus_ob_id is not None:
            self.bus_ob_id = bus_ob_id
        if bus_ob_name is not None:
            self.bus_ob_name = bus_ob_name
        if bus_ob_public_id is not None:
            self.bus_ob_public_id = bus_ob_public_id
        if bus_ob_rec_id is not None:
            self.bus_ob_rec_id = bus_ob_rec_id
        if include_links is not None:
            self.include_links = include_links
        if types is not None:
            self.types = types

    @property
    def attachment_id(self):
        """Gets the attachment_id of this AttachmentsRequest.  # noqa: E501


        :return: The attachment_id of this AttachmentsRequest.  # noqa: E501
        :rtype: str
        """
        return self._attachment_id

    @attachment_id.setter
    def attachment_id(self, attachment_id):
        """Sets the attachment_id of this AttachmentsRequest.


        :param attachment_id: The attachment_id of this AttachmentsRequest.  # noqa: E501
        :type: str
        """

        self._attachment_id = attachment_id

    @property
    def attachment_types(self):
        """Gets the attachment_types of this AttachmentsRequest.  # noqa: E501


        :return: The attachment_types of this AttachmentsRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._attachment_types

    @attachment_types.setter
    def attachment_types(self, attachment_types):
        """Sets the attachment_types of this AttachmentsRequest.


        :param attachment_types: The attachment_types of this AttachmentsRequest.  # noqa: E501
        :type: list[str]
        """
        allowed_values = ["Imported", "Linked", "URL"]  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                not set(attachment_types).issubset(set(allowed_values))):  # noqa: E501
            raise ValueError(
                "Invalid values for `attachment_types` [{0}], must be a subset of [{1}]"  # noqa: E501
                .format(", ".join(map(str, set(attachment_types) - set(allowed_values))),  # noqa: E501
                        ", ".join(map(str, allowed_values)))
            )

        self._attachment_types = attachment_types

    @property
    def bus_ob_id(self):
        """Gets the bus_ob_id of this AttachmentsRequest.  # noqa: E501


        :return: The bus_ob_id of this AttachmentsRequest.  # noqa: E501
        :rtype: str
        """
        return self._bus_ob_id

    @bus_ob_id.setter
    def bus_ob_id(self, bus_ob_id):
        """Sets the bus_ob_id of this AttachmentsRequest.


        :param bus_ob_id: The bus_ob_id of this AttachmentsRequest.  # noqa: E501
        :type: str
        """

        self._bus_ob_id = bus_ob_id

    @property
    def bus_ob_name(self):
        """Gets the bus_ob_name of this AttachmentsRequest.  # noqa: E501


        :return: The bus_ob_name of this AttachmentsRequest.  # noqa: E501
        :rtype: str
        """
        return self._bus_ob_name

    @bus_ob_name.setter
    def bus_ob_name(self, bus_ob_name):
        """Sets the bus_ob_name of this AttachmentsRequest.


        :param bus_ob_name: The bus_ob_name of this AttachmentsRequest.  # noqa: E501
        :type: str
        """

        self._bus_ob_name = bus_ob_name

    @property
    def bus_ob_public_id(self):
        """Gets the bus_ob_public_id of this AttachmentsRequest.  # noqa: E501


        :return: The bus_ob_public_id of this AttachmentsRequest.  # noqa: E501
        :rtype: str
        """
        return self._bus_ob_public_id

    @bus_ob_public_id.setter
    def bus_ob_public_id(self, bus_ob_public_id):
        """Sets the bus_ob_public_id of this AttachmentsRequest.


        :param bus_ob_public_id: The bus_ob_public_id of this AttachmentsRequest.  # noqa: E501
        :type: str
        """

        self._bus_ob_public_id = bus_ob_public_id

    @property
    def bus_ob_rec_id(self):
        """Gets the bus_ob_rec_id of this AttachmentsRequest.  # noqa: E501


        :return: The bus_ob_rec_id of this AttachmentsRequest.  # noqa: E501
        :rtype: str
        """
        return self._bus_ob_rec_id

    @bus_ob_rec_id.setter
    def bus_ob_rec_id(self, bus_ob_rec_id):
        """Sets the bus_ob_rec_id of this AttachmentsRequest.


        :param bus_ob_rec_id: The bus_ob_rec_id of this AttachmentsRequest.  # noqa: E501
        :type: str
        """

        self._bus_ob_rec_id = bus_ob_rec_id

    @property
    def include_links(self):
        """Gets the include_links of this AttachmentsRequest.  # noqa: E501


        :return: The include_links of this AttachmentsRequest.  # noqa: E501
        :rtype: bool
        """
        return self._include_links

    @include_links.setter
    def include_links(self, include_links):
        """Sets the include_links of this AttachmentsRequest.


        :param include_links: The include_links of this AttachmentsRequest.  # noqa: E501
        :type: bool
        """

        self._include_links = include_links

    @property
    def types(self):
        """Gets the types of this AttachmentsRequest.  # noqa: E501


        :return: The types of this AttachmentsRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._types

    @types.setter
    def types(self, types):
        """Sets the types of this AttachmentsRequest.


        :param types: The types of this AttachmentsRequest.  # noqa: E501
        :type: list[str]
        """
        allowed_values = ["None", "File", "FileManagerFile", "BusOb", "History", "Other"]  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                not set(types).issubset(set(allowed_values))):  # noqa: E501
            raise ValueError(
                "Invalid values for `types` [{0}], must be a subset of [{1}]"  # noqa: E501
                .format(", ".join(map(str, set(types) - set(allowed_values))),  # noqa: E501
                        ", ".join(map(str, allowed_values)))
            )

        self._types = types

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AttachmentsRequest):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, AttachmentsRequest):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Cherwell REST API

    Unofficial Python Cherwell REST API library.  # noqa: E501

    The version of the OpenAPI document: 9.3.2
    Contact: See AUTHORS.
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from pycherwell.configuration import Configuration


class RelatedBusinessObjectResponse(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'error_code': 'str',
        'error_message': 'str',
        'has_error': 'bool',
        'links': 'list[Link]',
        'page_number': 'int',
        'page_size': 'int',
        'parent_bus_ob_id': 'str',
        'parent_bus_ob_public_id': 'str',
        'parent_bus_ob_rec_id': 'str',
        'related_business_objects': 'list[ReadResponse]',
        'relationship_id': 'str',
        'total_records': 'int'
    }

    attribute_map = {
        'error_code': 'errorCode',
        'error_message': 'errorMessage',
        'has_error': 'hasError',
        'links': 'links',
        'page_number': 'pageNumber',
        'page_size': 'pageSize',
        'parent_bus_ob_id': 'parentBusObId',
        'parent_bus_ob_public_id': 'parentBusObPublicId',
        'parent_bus_ob_rec_id': 'parentBusObRecId',
        'related_business_objects': 'relatedBusinessObjects',
        'relationship_id': 'relationshipId',
        'total_records': 'totalRecords'
    }

    def __init__(self, error_code=None, error_message=None, has_error=None, links=None, page_number=None, page_size=None, parent_bus_ob_id=None, parent_bus_ob_public_id=None, parent_bus_ob_rec_id=None, related_business_objects=None, relationship_id=None, total_records=None, local_vars_configuration=None):  # noqa: E501
        """RelatedBusinessObjectResponse - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._error_code = None
        self._error_message = None
        self._has_error = None
        self._links = None
        self._page_number = None
        self._page_size = None
        self._parent_bus_ob_id = None
        self._parent_bus_ob_public_id = None
        self._parent_bus_ob_rec_id = None
        self._related_business_objects = None
        self._relationship_id = None
        self._total_records = None
        self.discriminator = None

        if error_code is not None:
            self.error_code = error_code
        if error_message is not None:
            self.error_message = error_message
        if has_error is not None:
            self.has_error = has_error
        if links is not None:
            self.links = links
        if page_number is not None:
            self.page_number = page_number
        if page_size is not None:
            self.page_size = page_size
        if parent_bus_ob_id is not None:
            self.parent_bus_ob_id = parent_bus_ob_id
        if parent_bus_ob_public_id is not None:
            self.parent_bus_ob_public_id = parent_bus_ob_public_id
        if parent_bus_ob_rec_id is not None:
            self.parent_bus_ob_rec_id = parent_bus_ob_rec_id
        if related_business_objects is not None:
            self.related_business_objects = related_business_objects
        if relationship_id is not None:
            self.relationship_id = relationship_id
        if total_records is not None:
            self.total_records = total_records

    @property
    def error_code(self):
        """Gets the error_code of this RelatedBusinessObjectResponse.  # noqa: E501


        :return: The error_code of this RelatedBusinessObjectResponse.  # noqa: E501
        :rtype: str
        """
        return self._error_code

    @error_code.setter
    def error_code(self, error_code):
        """Sets the error_code of this RelatedBusinessObjectResponse.


        :param error_code: The error_code of this RelatedBusinessObjectResponse.  # noqa: E501
        :type: str
        """

        self._error_code = error_code

    @property
    def error_message(self):
        """Gets the error_message of this RelatedBusinessObjectResponse.  # noqa: E501


        :return: The error_message of this RelatedBusinessObjectResponse.  # noqa: E501
        :rtype: str
        """
        return self._error_message

    @error_message.setter
    def error_message(self, error_message):
        """Sets the error_message of this RelatedBusinessObjectResponse.


        :param error_message: The error_message of this RelatedBusinessObjectResponse.  # noqa: E501
        :type: str
        """

        self._error_message = error_message

    @property
    def has_error(self):
        """Gets the has_error of this RelatedBusinessObjectResponse.  # noqa: E501


        :return: The has_error of this RelatedBusinessObjectResponse.  # noqa: E501
        :rtype: bool
        """
        return self._has_error

    @has_error.setter
    def has_error(self, has_error):
        """Sets the has_error of this RelatedBusinessObjectResponse.


        :param has_error: The has_error of this RelatedBusinessObjectResponse.  # noqa: E501
        :type: bool
        """

        self._has_error = has_error

    @property
    def links(self):
        """Gets the links of this RelatedBusinessObjectResponse.  # noqa: E501


        :return: The links of this RelatedBusinessObjectResponse.  # noqa: E501
        :rtype: list[Link]
        """
        return self._links

    @links.setter
    def links(self, links):
        """Sets the links of this RelatedBusinessObjectResponse.


        :param links: The links of this RelatedBusinessObjectResponse.  # noqa: E501
        :type: list[Link]
        """

        self._links = links

    @property
    def page_number(self):
        """Gets the page_number of this RelatedBusinessObjectResponse.  # noqa: E501


        :return: The page_number of this RelatedBusinessObjectResponse.  # noqa: E501
        :rtype: int
        """
        return self._page_number

    @page_number.setter
    def page_number(self, page_number):
        """Sets the page_number of this RelatedBusinessObjectResponse.


        :param page_number: The page_number of this RelatedBusinessObjectResponse.  # noqa: E501
        :type: int
        """

        self._page_number = page_number

    @property
    def page_size(self):
        """Gets the page_size of this RelatedBusinessObjectResponse.  # noqa: E501


        :return: The page_size of this RelatedBusinessObjectResponse.  # noqa: E501
        :rtype: int
        """
        return self._page_size

    @page_size.setter
    def page_size(self, page_size):
        """Sets the page_size of this RelatedBusinessObjectResponse.


        :param page_size: The page_size of this RelatedBusinessObjectResponse.  # noqa: E501
        :type: int
        """

        self._page_size = page_size

    @property
    def parent_bus_ob_id(self):
        """Gets the parent_bus_ob_id of this RelatedBusinessObjectResponse.  # noqa: E501


        :return: The parent_bus_ob_id of this RelatedBusinessObjectResponse.  # noqa: E501
        :rtype: str
        """
        return self._parent_bus_ob_id

    @parent_bus_ob_id.setter
    def parent_bus_ob_id(self, parent_bus_ob_id):
        """Sets the parent_bus_ob_id of this RelatedBusinessObjectResponse.


        :param parent_bus_ob_id: The parent_bus_ob_id of this RelatedBusinessObjectResponse.  # noqa: E501
        :type: str
        """

        self._parent_bus_ob_id = parent_bus_ob_id

    @property
    def parent_bus_ob_public_id(self):
        """Gets the parent_bus_ob_public_id of this RelatedBusinessObjectResponse.  # noqa: E501


        :return: The parent_bus_ob_public_id of this RelatedBusinessObjectResponse.  # noqa: E501
        :rtype: str
        """
        return self._parent_bus_ob_public_id

    @parent_bus_ob_public_id.setter
    def parent_bus_ob_public_id(self, parent_bus_ob_public_id):
        """Sets the parent_bus_ob_public_id of this RelatedBusinessObjectResponse.


        :param parent_bus_ob_public_id: The parent_bus_ob_public_id of this RelatedBusinessObjectResponse.  # noqa: E501
        :type: str
        """

        self._parent_bus_ob_public_id = parent_bus_ob_public_id

    @property
    def parent_bus_ob_rec_id(self):
        """Gets the parent_bus_ob_rec_id of this RelatedBusinessObjectResponse.  # noqa: E501


        :return: The parent_bus_ob_rec_id of this RelatedBusinessObjectResponse.  # noqa: E501
        :rtype: str
        """
        return self._parent_bus_ob_rec_id

    @parent_bus_ob_rec_id.setter
    def parent_bus_ob_rec_id(self, parent_bus_ob_rec_id):
        """Sets the parent_bus_ob_rec_id of this RelatedBusinessObjectResponse.


        :param parent_bus_ob_rec_id: The parent_bus_ob_rec_id of this RelatedBusinessObjectResponse.  # noqa: E501
        :type: str
        """

        self._parent_bus_ob_rec_id = parent_bus_ob_rec_id

    @property
    def related_business_objects(self):
        """Gets the related_business_objects of this RelatedBusinessObjectResponse.  # noqa: E501


        :return: The related_business_objects of this RelatedBusinessObjectResponse.  # noqa: E501
        :rtype: list[ReadResponse]
        """
        return self._related_business_objects

    @related_business_objects.setter
    def related_business_objects(self, related_business_objects):
        """Sets the related_business_objects of this RelatedBusinessObjectResponse.


        :param related_business_objects: The related_business_objects of this RelatedBusinessObjectResponse.  # noqa: E501
        :type: list[ReadResponse]
        """

        self._related_business_objects = related_business_objects

    @property
    def relationship_id(self):
        """Gets the relationship_id of this RelatedBusinessObjectResponse.  # noqa: E501


        :return: The relationship_id of this RelatedBusinessObjectResponse.  # noqa: E501
        :rtype: str
        """
        return self._relationship_id

    @relationship_id.setter
    def relationship_id(self, relationship_id):
        """Sets the relationship_id of this RelatedBusinessObjectResponse.


        :param relationship_id: The relationship_id of this RelatedBusinessObjectResponse.  # noqa: E501
        :type: str
        """

        self._relationship_id = relationship_id

    @property
    def total_records(self):
        """Gets the total_records of this RelatedBusinessObjectResponse.  # noqa: E501


        :return: The total_records of this RelatedBusinessObjectResponse.  # noqa: E501
        :rtype: int
        """
        return self._total_records

    @total_records.setter
    def total_records(self, total_records):
        """Sets the total_records of this RelatedBusinessObjectResponse.


        :param total_records: The total_records of this RelatedBusinessObjectResponse.  # noqa: E501
        :type: int
        """

        self._total_records = total_records

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RelatedBusinessObjectResponse):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, RelatedBusinessObjectResponse):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Cherwell REST API

    Unofficial Python Cherwell REST API library.  # noqa: E501

    The version of the OpenAPI document: 9.3.2
    Contact: See AUTHORS.
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from pycherwell.configuration import Configuration


class ServiceInfoResponse(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'api_version': 'str',
        'csm_culture': 'str',
        'csm_version': 'str',
        'system_date_time': 'datetime',
        'time_zone': 'object'
    }

    attribute_map = {
        'api_version': 'apiVersion',
        'csm_culture': 'csmCulture',
        'csm_version': 'csmVersion',
        'system_date_time': 'systemDateTime',
        'time_zone': 'timeZone'
    }

    def __init__(self, api_version=None, csm_culture=None, csm_version=None, system_date_time=None, time_zone=None, local_vars_configuration=None):  # noqa: E501
        """ServiceInfoResponse - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._api_version = None
        self._csm_culture = None
        self._csm_version = None
        self._system_date_time = None
        self._time_zone = None
        self.discriminator = None

        if api_version is not None:
            self.api_version = api_version
        if csm_culture is not None:
            self.csm_culture = csm_culture
        if csm_version is not None:
            self.csm_version = csm_version
        if system_date_time is not None:
            self.system_date_time = system_date_time
        if time_zone is not None:
            self.time_zone = time_zone

    @property
    def api_version(self):
        """Gets the api_version of this ServiceInfoResponse.  # noqa: E501


        :return: The api_version of this ServiceInfoResponse.  # noqa: E501
        :rtype: str
        """
        return self._api_version

    @api_version.setter
    def api_version(self, api_version):
        """Sets the api_version of this ServiceInfoResponse.


        :param api_version: The api_version of this ServiceInfoResponse.  # noqa: E501
        :type: str
        """

        self._api_version = api_version

    @property
    def csm_culture(self):
        """Gets the csm_culture of this ServiceInfoResponse.  # noqa: E501


        :return: The csm_culture of this ServiceInfoResponse.  # noqa: E501
        :rtype: str
        """
        return self._csm_culture

    @csm_culture.setter
    def csm_culture(self, csm_culture):
        """Sets the csm_culture of this ServiceInfoResponse.


        :param csm_culture: The csm_culture of this ServiceInfoResponse.  # noqa: E501
        :type: str
        """

        self._csm_culture = csm_culture

    @property
    def csm_version(self):
        """Gets the csm_version of this ServiceInfoResponse.  # noqa: E501


        :return: The csm_version of this ServiceInfoResponse.  # noqa: E501
        :rtype: str
        """
        return self._csm_version

    @csm_version.setter
    def csm_version(self, csm_version):
        """Sets the csm_version of this ServiceInfoResponse.


        :param csm_version: The csm_version of this ServiceInfoResponse.  # noqa: E501
        :type: str
        """

        self._csm_version = csm_version

    @property
    def system_date_time(self):
        """Gets the system_date_time of this ServiceInfoResponse.  # noqa: E501


        :return: The system_date_time of this ServiceInfoResponse.  # noqa: E501
        :rtype: datetime
        """
        return self._system_date_time

    @system_date_time.setter
    def system_date_time(self, system_date_time):
        """Sets the system_date_time of this ServiceInfoResponse.


        :param system_date_time: The system_date_time of this ServiceInfoResponse.  # noqa: E501
        :type: datetime
        """

        self._system_date_time = system_date_time

    @property
    def time_zone(self):
        """Gets the time_zone of this ServiceInfoResponse.  # noqa: E501


        :return: The time_zone of this ServiceInfoResponse.  # noqa: E501
        :rtype: object
        """
        return self._time_zone

    @time_zone.setter
    def time_zone(self, time_zone):
        """Sets the time_zone of this ServiceInfoResponse.


        :param time_zone: The time_zone of this ServiceInfoResponse.  # noqa: E501
        :type: object
        """

        self._time_zone = time_zone

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ServiceInfoResponse):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ServiceInfoResponse):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Cherwell REST API

    Unofficial Python Cherwell REST API library.  # noqa: E501

    The version of the OpenAPI document: 9.3.2
    Contact: See AUTHORS.
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from pycherwell.configuration import Configuration


class Section(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'section_fields': 'list[SectionField]',
        'gallery_image': 'str',
        'title': 'str',
        'relationship_id': 'str',
        'target_bus_ob_id': 'str',
        'target_bus_ob_rec_id': 'str'
    }

    attribute_map = {
        'section_fields': 'sectionFields',
        'gallery_image': 'galleryImage',
        'title': 'title',
        'relationship_id': 'relationshipId',
        'target_bus_ob_id': 'targetBusObId',
        'target_bus_ob_rec_id': 'targetBusObRecId'
    }

    def __init__(self, section_fields=None, gallery_image=None, title=None, relationship_id=None, target_bus_ob_id=None, target_bus_ob_rec_id=None, local_vars_configuration=None):  # noqa: E501
        """Section - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._section_fields = None
        self._gallery_image = None
        self._title = None
        self._relationship_id = None
        self._target_bus_ob_id = None
        self._target_bus_ob_rec_id = None
        self.discriminator = None

        if section_fields is not None:
            self.section_fields = section_fields
        if gallery_image is not None:
            self.gallery_image = gallery_image
        if title is not None:
            self.title = title
        if relationship_id is not None:
            self.relationship_id = relationship_id
        if target_bus_ob_id is not None:
            self.target_bus_ob_id = target_bus_ob_id
        if target_bus_ob_rec_id is not None:
            self.target_bus_ob_rec_id = target_bus_ob_rec_id

    @property
    def section_fields(self):
        """Gets the section_fields of this Section.  # noqa: E501


        :return: The section_fields of this Section.  # noqa: E501
        :rtype: list[SectionField]
        """
        return self._section_fields

    @section_fields.setter
    def section_fields(self, section_fields):
        """Sets the section_fields of this Section.


        :param section_fields: The section_fields of this Section.  # noqa: E501
        :type: list[SectionField]
        """

        self._section_fields = section_fields

    @property
    def gallery_image(self):
        """Gets the gallery_image of this Section.  # noqa: E501


        :return: The gallery_image of this Section.  # noqa: E501
        :rtype: str
        """
        return self._gallery_image

    @gallery_image.setter
    def gallery_image(self, gallery_image):
        """Sets the gallery_image of this Section.


        :param gallery_image: The gallery_image of this Section.  # noqa: E501
        :type: str
        """

        self._gallery_image = gallery_image

    @property
    def title(self):
        """Gets the title of this Section.  # noqa: E501


        :return: The title of this Section.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this Section.


        :param title: The title of this Section.  # noqa: E501
        :type: str
        """

        self._title = title

    @property
    def relationship_id(self):
        """Gets the relationship_id of this Section.  # noqa: E501


        :return: The relationship_id of this Section.  # noqa: E501
        :rtype: str
        """
        return self._relationship_id

    @relationship_id.setter
    def relationship_id(self, relationship_id):
        """Sets the relationship_id of this Section.


        :param relationship_id: The relationship_id of this Section.  # noqa: E501
        :type: str
        """

        self._relationship_id = relationship_id

    @property
    def target_bus_ob_id(self):
        """Gets the target_bus_ob_id of this Section.  # noqa: E501


        :return: The target_bus_ob_id of this Section.  # noqa: E501
        :rtype: str
        """
        return self._target_bus_ob_id

    @target_bus_ob_id.setter
    def target_bus_ob_id(self, target_bus_ob_id):
        """Sets the target_bus_ob_id of this Section.


        :param target_bus_ob_id: The target_bus_ob_id of this Section.  # noqa: E501
        :type: str
        """

        self._target_bus_ob_id = target_bus_ob_id

    @property
    def target_bus_ob_rec_id(self):
        """Gets the target_bus_ob_rec_id of this Section.  # noqa: E501


        :return: The target_bus_ob_rec_id of this Section.  # noqa: E501
        :rtype: str
        """
        return self._target_bus_ob_rec_id

    @target_bus_ob_rec_id.setter
    def target_bus_ob_rec_id(self, target_bus_ob_rec_id):
        """Sets the target_bus_ob_rec_id of this Section.


        :param target_bus_ob_rec_id: The target_bus_ob_rec_id of this Section.  # noqa: E501
        :type: str
        """

        self._target_bus_ob_rec_id = target_bus_ob_rec_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Section):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Section):
            return True

        return self.to_dict() != other.to_dict()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Copyright (C) 2019 Paul Greenberg @greenpau
See LICENSE for licensing details
'''

from __future__ import (absolute_import, division, print_function)
import os
import sys
import stat
import re
import json
import yaml
import copy
import logging
import textwrap
from datetime import datetime, timedelta
from pprint import pprint

# python 2 and python 3 compatibility library
import six
from six.moves.urllib.parse import quote

from pycherwell import ApiClient, Configuration, ApiException
from pycherwell import BusinessObjectApi, TemplateRequest, SaveRequest, FieldTemplateItem
from pycherwell import SearchesApi, SearchResultsRequest, FilterInfo
from pycherwell import TeamsApi
from pycherwell import ServiceApi
from pycherwell.app_configuration import AppConfiguration
import urllib3
urllib3.disable_warnings()
import requests
try:
    import http.client as http_client
except ImportError:
    # Python 2
    import httplib as http_client

__author__ = "Paul Greenberg @greenpau"
__version__ = '1.0.9'
__maintainer__ = "Paul Greenberg"
__email__ = "greenpau@outlook.com"
__status__ = "Alpha"

class CherwellClient(object):
    '''
    This class implements the client application.
    '''

    def __init__(self):
        ''' Initializes the class. '''
        self.log = logging.getLogger('cherwell-client')
        handler = logging.StreamHandler(sys.stderr)
        formatter = logging.Formatter('[%(asctime)s] %(levelname)s %(name)s@%(lineno)d: %(message)s')
        handler.setFormatter(formatter)
        self.log.addHandler(handler)
        self.debug_enabled = False
        self.output_format = 'json'
        self.api_configuration = None
        self.api_client = None
        self.profile='default'
        self.config_file=None
        self._is_configured = False
        self.auth_token = None
        self.user_agent = None
        self._authenticated = False
        return

    def set_config_file(self, config_file):
        self.config_file = config_file

    def set_profile(self, profile='default'):
        self.profile = profile

    def _configure(self):
        self.config = AppConfiguration(self.config_file, self.profile)
        if self.debug_enabled:
            self.config.log.setLevel(logging.DEBUG)
            self.config.debug_enabled = True
        self.config.load()
        self.api_configuration = Configuration()
        if self.debug_enabled:
            self.api_configuration.debug = True
        self.api_configuration.verify_ssl = self.config.get_verify_ssl()
        self.api_configuration.ssl_ca_cert = self.config.get_ssl_ca_cert()
        self.api_configuration.assert_hostname = self.config.get_assert_hostname()
        self.api_configuration.connection_pool_maxsize = self.config.get_connection_pool_maxsize()
        self.api_configuration.proxy = self.config.get_proxy()
        self.api_configuration.cert_file = self.config.get_cert_file()
        self.api_configuration.key_file = self.config.get_key_file()
        self.api_configuration.safe_chars_for_path_param = self.config.get_safe_chars_for_path_param()
        self.api_configuration.host = self.config.get_host()
        self.user_agent = self.config.get_user_agent()
        return

    def _enable(self, opts={}):
        if not self._is_configured:
            self._configure()
            self._is_configured = True
        if not self.auth_token:
            self._authenticate()
        if not self._authenticated:
            raise Exception('client', 'not authenticated')
        self.api_client = ApiClient(
            configuration = self.api_configuration,
            header_name=self.config.get_auth_header_name(),
            header_value=self.config.get_auth_header_value(),
        )
        self.api_client.user_agent = self.config.get_user_agent()
        app_sections = [
            'business_objects',
            'teams',
        ]
        for app_section in app_sections:
            loaded = self.config.load_app_section(app_section)
            self.log.debug('the %s section data was not loaded from cache', app_section)
        return

    def _ensure_required_business_objects(self, app_sections=[], opts={}):
        for app_section in app_sections:
            self.log.debug('ensuring required section exists in cache: %s', app_section)
            if self.config.app[app_section]:
                continue
            if app_section == "business_objects":
                self.get_business_object_summaries({
                    'save_app_section': True,
                    'summary_type': 'All'
                })
            elif app_section == "teams":
                self.get_teams()
            else:
                raise Exception('client', '_ensure_required_business_objects does not support %s section' % (app_section))

            loaded = self.config.load_app_section(app_section)
            if not loaded:
                raise Exception('client', '_ensure_required_business_objects() failed to load %s section' % (app_section))

    def _ensure_required_business_object_fields(self, business_objects=[], opts={}):
        for business_object_name in business_objects:
            self.log.debug('ensuring business object fields exist in cache: %s', business_object_name)
            business_object_id = self.config.get_business_object_id(business_object_name)
            if not business_object_id:
                raise Exception('client', 'failed finding ID for BO %s' % (business_object_name))

            business_object_fields = self.get_business_object_fields({'business_object_id': business_object_id})
            if not business_object_fields:
                raise Exception('client', '_ensure_required_business_object_fields() failed for BO %s' % (business_object_name))
        return

    def _ensure_required_business_object_templates(self, business_objects=[], opts={}):
        for business_object_name in business_objects:
            self.log.debug('ensuring business object templates exist in cache: %s', business_object_name)
            business_object_id = self.config.get_business_object_id(business_object_name)
            if not business_object_id:
                raise Exception('client', 'failed finding ID for BO %s' % (business_object_name))

            business_object_template = self.get_business_object_template({'business_object_id': business_object_id})
            if not business_object_template:
                raise Exception('client', '_ensure_required_business_object_templates() failed for BO %s' % (business_object_name))
        return

    def debug(self):
        if self.debug_enabled:
            return
        http_client.HTTPConnection.debuglevel = 5
        self.log.setLevel(logging.DEBUG)
        self.debug_enabled = True
        return

    def set_output_format(self, output_format='json'):
        self.output_format = output_format

    def get_version(self):
        return __version__

    @staticmethod
    def _wrap_return(opts, container, data):
        if 'without_header' in opts:
            return data
        return {container: data}

    def get_service_info(self, opts={}):
        data = {}
        api_response = None
        self._enable()
        try:
            api_instance = ServiceApi(self.api_client)
            api_response = api_instance.service_get_service_info_v1()
        except ApiException as e:
            self.log.error('Exception when calling ServiceApi->service_get_service_info_v1: %s', e)
            return data
        '''
        data['api_version'] = api_response.api_version
        data['csm_culture'] = api_response.csm_culture
        data['csm_version'] = api_response.csm_version
        data['system_date_time'] = str(api_response.system_date_time)
        '''
        data = api_response.to_dict()
        data['system_date_time'] = str(data['system_date_time'])
        return self._wrap_return(opts, 'service_info', data)

    def get_business_object_summaries(self, opts={}):
        data = []
        api_response = None
        summary_type = 'Major'
        if 'summary_type' in opts:
            if opts['summary_type'] not in ['All', 'Major', 'Lookup', 'Supporting', 'Groups']:
                raise Exception('client', 'Unsupported summary type: %s' % (opts['summary_type']))
            summary_type = opts['summary_type']
        self._enable()
        try:
            api_instance = BusinessObjectApi(self.api_client)
            api_response = api_instance.business_object_get_business_object_summaries_v1(summary_type)
        except ApiException as e:
            self.log.error('Exception when calling BusinessObjectApi->business_object_get_business_object_summaries_v1: %s', e)
            return data
        for entry in api_response:
            data.append(entry.to_dict())
        if 'save_app_section' in opts and summary_type == 'All':
            self.config.save_app_section('business_objects', data)
        return self._wrap_return(opts, 'business_object_summaries', data)


    def get_business_object_fields(self, opts={}):
        self._enable()
        if 'business_object_id' not in opts:
            raise Exception('client', 'get_business_object_fields() requires business_object_id')
        business_object_id = opts['business_object_id']
        if 'business_object_fields_ref' in self.config.app:
            if business_object_id in self.config.app['business_object_fields_ref']:
                return self.config.app['business_object_fields_ref'][business_object_id]
        loaded = self.config.load_app_data('business_object_fields_ref', business_object_id)
        if loaded:
            return self.config.app['business_object_fields_ref'][business_object_id]
        api_response = None
        try:
            api_instance = BusinessObjectApi(self.api_client)
            api_response = api_instance.business_object_get_business_object_schema_v1(business_object_id)
        except ApiException as e:
            self.log.error('Exception when calling BusinessObjectApi->business_object_get_business_object_summaries_v1: %s', e)
            return False
        api_response = api_response.to_dict()
        if 'field_definitions' not in api_response:
            err = 'field_definitions not found in API response'
            self.log.error('Exception when calling BusinessObjectApi->business_object_get_business_object_summaries_v1: %s', err)
            return False

        self.config.save_app_data('business_object_fields_ref', business_object_id, api_response['field_definitions'])
        return self.config.app['business_object_fields_ref'][business_object_id]

    def get_business_object_template(self, opts={}):
        self._enable()
        if 'business_object_id' not in opts:
            raise Exception('client', 'get_business_object_template() requires business_object_id')
        business_object_id = opts['business_object_id']
        if 'business_object_template_ref' in self.config.app:
            if business_object_id in self.config.app['business_object_template_ref']:
                return self.config.app['business_object_template_ref'][business_object_id]
        loaded = self.config.load_app_data('business_object_template_ref', business_object_id)
        if loaded:
            return self.config.app['business_object_template_ref'][business_object_id]

        template_fields = None
        required_template_fields = {}

        for i in [1, 2]:
            api_response = None
            kwargs = {
                'bus_ob_id': business_object_id,
                'include_all': True,
                'include_required': True,
            }
            if i == 2:
                kwargs['include_all'] = False
            try:
                template_request = TemplateRequest(**kwargs)
                api_instance = BusinessObjectApi(self.api_client)
                api_response = api_instance.business_object_get_business_object_template_v1(template_request)
            except ApiException as e:
                self.log.error('Exception when calling BusinessObjectApi->business_object_get_business_object_template_v1: %s', e)
                return False
            api_response = api_response.to_dict()
            if 'fields' not in api_response:
                err = 'fields not found in API response'
                self.log.error('Exception when calling BusinessObjectApi->business_object_get_business_object_template_v1: %s', err)
                return False

            if i == 1:
                template_fields = api_response['fields']
                continue

            for field in api_response['fields']:
                if 'field_id' not in field:
                    continue
                required_template_fields[field['field_id']] = True

        for i, field in enumerate(template_fields):
            if 'field_id' not in field:
                continue
            if field['field_id'] in required_template_fields:
                template_fields[i]['required'] = True
            else:
                template_fields[i]['required'] = False

        self.config.save_app_data('business_object_template_ref', business_object_id, template_fields)
        return self.config.app['business_object_template_ref'][business_object_id]

    def get_incident(self, opts={}):
        """lookup an incident by human visible id (publicid).

        Note: only a single dict is returned.

        Args:
            opts: List of options to perform lookup.

        Returns:
            returns a single dict containing the incident information.

        """

        api_response = None
        if 'incident_id' not in opts:
            raise Exception('client', 'incident_id not found')
        incident_id = str(opts['incident_id'])
        self._enable()
        self._ensure_required_business_objects(['business_objects'])
        self._ensure_required_business_object_fields(['Incident'], opts)

        busObId = self.config.get_business_object_id('Incident')
        if not busObId:
            raise Exception('internal','Failed to find business object ID of Incident Type')
        
        apiName = 'BusinessObjectApi'
        apiFunc = 'business_object_get_business_object_by_public_id_v1'

        try:
            api_instance = BusinessObjectApi(self.api_client)
            api_response = getattr(api_instance, apiFunc)(busObId, incident_id)
            api_response = api_response.to_dict()
        except ApiException as e:
            if 'RECORDNOTFOUND' in str(e):
                self.log.warn("Incident ID %s not found", incident_id)
                return
            self.log.error('Exception when calling %s->%s: %s', apiName, apiFunc, e)
            return

        if 'bus_ob_public_id' not in api_response:
            self.log.warn("Incident ID %s not found", incident_id)
            return

        if 'output_format' in opts and opts['output_format'] in ['csv', 'text']:
            rows = []
            if 'fields' not in api_response:
                self.log.warn("Incident ID %s has no data: %s", incident_id, api_response)
                return
            if opts['output_format'] == 'text':
                incident_type = self._get_field_value(api_response['fields'], ('display_name', 'value', 'Incident Type'))
                rows.append(['', '%s # %s' % (incident_type, incident_id)])
                rows.extend(self._serialize_kv(api_response['fields'], ('display_name', 'value'), opts))
            elif opts['output_format'] == 'csv':
                rows.append(self._get_csv_headers(api_response['fields'], 'display_name'))
                rows.append(self._get_csv_columns(api_response['fields'], api_response['fields']))
            else:
                pass
            return rows
        return api_response

    def get_teams(self, opts={}):
        teams = []
        api_response = None
        self._enable()
        try:
            api_instance = TeamsApi(self.api_client)
            api_response = api_instance.teams_get_teams_v2()
        except ApiException as e:
            self.log.error('Exception when calling TeamsApi->teams_get_teams_v2: %s', e)
            return teams
        api_response = api_response.to_dict()
        if 'teams' not in api_response:
            self.log.error('The response from TeamsApi->teams_get_teams_v2() did not contain teams key')
            return teams
        if len(api_response['teams']) == 0:
            self.log.error('The response from TeamsApi->teams_get_teams_v2() had no teams')
            return teams

        if 'output_format' in opts and opts['output_format'] in ['csv', 'text']:
            teams.append(['Team Name', 'UUID'])

        for team in api_response['teams']:
            if 'output_format' in opts and opts['output_format'] in ['csv', 'text']:
                try:
                    teams.append([team['team_name'], team['team_id']])
                except:
                    pass
            else:
                teams.append(team)
        self.config.save_app_section('teams', api_response['teams'])
        return teams

    def get_journal(self, opts={}):
        api_response = None
        if 'incident_id' not in opts:
            raise Exception('client', 'incident_id not found')
        incident_id = str(opts['incident_id'])
        self._enable()
        self._ensure_required_business_objects(['business_objects'])
        self._ensure_required_business_object_fields(['Incident', 'Journal'], opts)

        busObId = self.config.get_business_object_id('Incident')
        if not busObId:
            raise Exception('internal', 'Failed to find business object ID of Incident Type')
        jbusObId = self.config.get_business_object_id('Journal')
        if not jbusObId:
             raise Exception('internal', 'Failed to find business object ID of Journal Type')
        self.log.debug("Get journal (%s) for incident %s (%s)", jbusObId, incident_id, busObId)

        apiName = 'BusinessObjectApi'
        apiFunc = 'business_object_get_business_object_by_public_id_v1'

        try:
            # First, get business object id
            api_instance = BusinessObjectApi(self.api_client)
            api_response = getattr(api_instance, apiFunc)(busObId, incident_id)
            api_response = api_response.to_dict()
            busRecId = api_response['bus_ob_rec_id']
            # Next, search by the business object id
            apiName = 'SearchesApi'
            apiFunc = 'searches_get_search_results_ad_hoc_v1'
            busObFilter = self._build_filter('RecID:eq:' + busRecId, busObId)
            busObFilters = [busObFilter]
            if 'search_conditions' in opts:
                for search_condition in opts['search_conditions']:
                    busObFilter = self._build_filter(search_condition, jbusObId)
                    if busObFilter:
                        busObFilters.append(busObFilter)
            kwargs = {
                'bus_ob_id': jbusObId,
                'filters': busObFilters,
                'include_all_fields': True,
            }
            search_request = SearchResultsRequest(**kwargs)
            api_instance = SearchesApi(self.api_client)
            api_response = getattr(api_instance, apiFunc)(search_request)
            api_response = api_response.to_dict()
            api_response = api_response['business_objects']
        except ApiException as e:
            if 'RECORDNOTFOUND' in str(e):
                self.log.warn("Incident ID %s not found", incident_id)
                return
            self.log.error('Exception when calling %s() via %s API: %s', apiFunc, apiName, e)
            return

        records = []
        record_fields = [
            'Created Date Time',
            'Created By',
            'Journal Type Name',
            'Details',
        ]
        for record in api_response:
            if 'fields' not in record:
                continue
            parsed_record = self._parse_record_fields(record['fields'], record_fields)
            records.append(parsed_record)

        if opts['output_format'] not in ['csv', 'text']:
            return records

        if 'output_format' in opts and opts['output_format'] in ['csv', 'text']:
            rows = []
            rows.append(record_fields)
            opts['column_width_allocation'] = [10, 10, 10, 70]
            if opts['output_format'] == 'text':
                column_width_allocation = self._get_column_width(record_fields, records, opts)
            for record in records:
                row = []
                for i, field in enumerate(record_fields):
                    if field in record:
                        v = record[field]
                        if opts['output_format'] == 'text':
                            if len(v) > column_width_allocation[i]:
                                v = '\n'.join(textwrap.wrap(v, width=column_width_allocation[i]))
                                v = v.replace('  ', '\n')
                        row.append(v)
                        continue
                    row.append('---')
                rows.append(row)
            return rows
        return records

    @staticmethod
    def _get_column_width(columns, entries, opts={}):
        width = {}
        max_width = 120
        if 'terminal_columns' in opts and opts['terminal_columns'] > 0:
            max_width = opts['terminal_columns'] - (len(columns) * 4)
        for i, column in enumerate(columns):
            column_width = max_width / 100 * opts['column_width_allocation'][i]
            width[i] = int(column_width)
        return width

    @staticmethod
    def _parse_record_fields(entries=[], fields=None):
        record = {}
        for entry in entries:
            if fields and entry['display_name'] not in fields:
                continue
            record[entry['display_name']] = entry['value']
        return record

    @staticmethod
    def _time_travel_snapshot(unit, count, measure='seconds'):
        if unit not in ['minute', 'hour', 'day', 'week', 'month', 'quarter', 'year']:
            return None
        if unit == 'minute':
            multiplier = 60
        elif unit == 'hour':
            multiplier = 3600
        elif unit == 'day':
            multiplier = 86400
        elif unit == 'week':
            multiplier = 604800
        elif unit == 'month':
            multiplier = 2.628e+6
        elif unit == 'quarter':
            multiplier = 7.884e+6
        elif unit == 'year':
            multiplier = 3.154e+7
        seconds = int(count) * multiplier
        return seconds

    def _build_filter(self, s, busObId):
        parts = s.split(':')
        if len(parts) < 3:
            raise Exception('parameters', 'invalid filter string: %s' % (s))
        field_id = parts[0]
        conditional = parts[1]
        value = None
        if len(parts) == 3:
            value = parts[2]
        else:
            value = ':'.join(parts[2:])
        if ' ago' in value:
            value = value.lower()
            m = re.match('(?P<count>\d+)\s+(?P<unit>minute|hour|day|week|month|quarter|year)[s]?\s+ago$', value)
            if not m:
                raise Exception('parameters', 'invalid filter string: %s' % (s))
            seconds = self._time_travel_snapshot(m.group('unit'), m.group('count'))
            if not seconds:
                raise Exception('parameters', 'invalid filter string: %s' % (s))
            ts = datetime.now() - timedelta(seconds=seconds)
            value = '%d/%d/%d 00:00:00 AM' % (ts.year, ts.month, ts.day)

        busObFieldId = self.config.get_business_object_field_id(busObId, field_id)
        kwargs = {
            'field_id': busObFieldId,
            'operator': conditional,
            'value': value,
        }
        return FilterInfo(**kwargs)

    def create_incident(self, opts={}):
        if 'create_as' not in opts:
            return {'error': 'no create-as found when creating the incident'}
        if 'create_fields' not in opts:
            return {'error': 'no create-fields found when creating the incident'}

        api_response = None
        self._enable()
        self._ensure_required_business_objects(['business_objects', 'teams'], opts)
        self._ensure_required_business_object_fields(['Incident', 'Customer'], opts)
        self._ensure_required_business_object_templates(['Incident'], opts)

        incidentBusObId = self.config.get_business_object_id('Incident')
        if not incidentBusObId:
            raise Exception('internal', 'Failed to find Incident business object ID')
        customerBusObId = self.config.get_business_object_id('Customer')
        if not customerBusObId:
            raise Exception('internal', 'Failed to find %s business object ID' % ('Customer'))
        incidentTemplate = self.get_business_object_template({'business_object_id': incidentBusObId})
        if not incidentTemplate:
            raise Exception('internal', 'Failed to find business object template for %s (%s)' % ('Incident', incidentBusObId))

        incidentFields = opts['create_fields']

        # customerBusOb = self.get_business_object('Customer', ...
        requestors = self.get_requestors({'search_conditions': opts['create_as']})
        if len(requestors) == 0:
            return {
                'error': 'requestor not found',
                'requestor_criteria': opts['create_as'],
            }
        elif len(requestors) > 1:
            return {
                'error': 'multiple requestors found',
                'requestor_criteria': opts['create_as'],
                'requestor_matches': requestors,
            }
        else:
            pass

        requestor = requestors[0]
        if  'bus_ob_rec_id' not in requestor:
            return {
                'error': 'requestor business object has no bus_ob_rec_id',
                'requestor': requestor,
            }

        incidentFields.append('Customer ID:%s' % (requestor['bus_ob_rec_id']))

        kwargs = {
            'bus_ob_id': incidentBusObId,
            'template': incidentTemplate,
            'fields': incidentFields,
        }
        result = self.create_business_object(**kwargs)

        return result

    def create_business_object(self, bus_ob_id=None, template=[], fields=[], opts={}):
        response = {}
        api_response = None
        if not bus_ob_id:
            raise Exception('internal', 'business object is None')
        if len(template) == 0:
            raise Exception('internal', 'business object template is empty')

        # Build input fields reference.
        input_fields = {}
        for entry in fields:
            parts = entry.split(':')
            if len(parts) != 2:
                if 'errors' not in response:
                    response['errors'] = []
                response['errors'].append('input field entry is invalid: %s' % (entry))
                continue
            if parts[0] in input_fields:
                if 'errors' not in response:
                    response['errors'] = []
                response['errors'].append('input field entry is duplicate: %s' % (entry))
                continue
            input_fields[parts[0]] = {'value': parts[1]}
        if 'Description' not in input_fields and 'ShortDescription' in input_fields:
            input_fields['Description'] = input_fields['ShortDescription']
        if 'ShortDescription' not in input_fields and 'Description' in input_fields:
            input_fields['ShortDescription'] = str(input_fields['Description']).split('\n')[0]

        if 'errors' in response:
            return response

        # Build template fields reference.
        template_fields = {}
        for entry in template:
            if 'field_id' not in entry:
                continue
            for k in ['name', 'display_name']:
                template_fields[entry[k]] = entry
            if 'required' in entry and entry['required'] == True:
                # required_found = False
                required_found = True
                for k in ['name', 'display_name']:
                    if entry[k] not in input_fields:
                        continue
                    required_found = True
                if not required_found:
                    if 'errors' not in response:
                        response['errors'] = []
                    response['errors'].append('required template field %s (%s) not found in input fields' % (entry['name'], entry['field_id']))

        # Check whether input fields are valid
        for k in input_fields:
            if k not in template_fields:
                if 'errors' not in response:
                    response['errors'] = []
                response['errors'].append('input field %s (%s) not found in template fields' % (k, input_fields[k]['value']))

        if 'errors' in response:
            return response

        # Build business object save request
        request_fields = []
        for k in input_fields:
            f = copy.deepcopy(template_fields[k])
            del f['required']
            f['value'] = input_fields[k]['value']
            f['field_id'] = 'BO:%s,FI:%s' % (bus_ob_id, template_fields[k]['field_id'])
            f['dirty'] = True
            ti = FieldTemplateItem(**f)
            request_fields.append(ti)

        # Perform request
        try:
            save_request = SaveRequest(bus_ob_id = bus_ob_id, fields=request_fields)
            api_instance = BusinessObjectApi(self.api_client)
            api_response = api_instance.business_object_save_business_object_v1(save_request)
        except ApiException as e:
            err_msg = None
            if 'errors' not in response:
                response['errors'] = []
            try:
                err_msg = json.loads(e.body)
            except:
                err_msg = 'Exception when calling BusinessObjectApi->business_object_save_business_object_v1: %s' % (e)
            response['errors'].append(err_msg)
            return response

        api_response = api_response.to_dict()
        for k in api_response:
            response[k] = api_response[k]
        return response

    def get_requestors(self, opts={}):
        response = {}
        if 'search_conditions' not in opts:
            raise Exception('internal', 'search conditions are required for searching requestor')
        api_response = None
        self._enable()
        self._ensure_required_business_objects(['business_objects'], opts)
        self._ensure_required_business_object_fields(['Customer'], opts)

        customerBusObId = self.config.get_business_object_id('Customer')
        if not customerBusObId:
            raise Exception('internal', 'Failed to find Customer business object ID')

        kwargs = {
            'bus_ob_id': customerBusObId,
        }

        busObFilters = []
        for search_condition in opts['search_conditions']:
            busObFilter = self._build_filter(search_condition, customerBusObId)
            if busObFilter:
                busObFilters.append(busObFilter)
        if not busObFilters:
            raise Exception('internal', 'failed to build search filters for Customer business object')

        kwargs = {
            'bus_ob_id': customerBusObId,
            'filters': busObFilters,
        }

        try:
            search_request = SearchResultsRequest(**kwargs)
            api_instance = SearchesApi(self.api_client)
            api_response = api_instance.searches_get_search_results_ad_hoc_v1(search_request)
        except ApiException as e:
            err_msg = None
            if 'errors' not in response:
                response['errors'] = []
            try:
                err_msg = json.loads(e.body)
            except:
                err_msg = 'Exception when calling SearchesApi->searches_get_search_results_ad_hoc_v1: %s' % (e)
            response['errors'].append(err_msg)
            return response

        api_response = api_response.to_dict()
        if 'business_objects' not in api_response:
            err_msg = 'Exception when calling SearchesApi->searches_get_search_results_ad_hoc_v1: business_objects not found'
            response['errors'] = [err_msg]
            return response

        return api_response['business_objects']


    def get_incidents(self, opts={}):
        incidents = []
        api_response = None
        self._enable()

        self._ensure_required_business_objects(['business_objects', 'teams'], opts)
        self._ensure_required_business_object_fields(['Incident'], opts)

        busObFields = []
        busObName = 'Incident'
        busObId = self.config.get_business_object_id(busObName)
        if not busObId:
            raise Exception('internal','Failed to find %s business object ID' % (busObName))

        kwargs = {'bus_ob_id': busObId}

        if 'search_conditions' in opts:
            busObFilters = []
            for search_condition in opts['search_conditions']:
                busObFilter = self._build_filter(search_condition, busObId)
                if busObFilter:
                    busObFilters.append(busObFilter)
            if busObFilters:
                kwargs['filters'] = busObFilters

        if 'search_fields' in opts:
            busObFieldIds = []
            for busObFieldName in opts['search_fields']:
                if busObFieldName not in busObFields:
                    busObFields.append(busObFieldName)
                busObFieldId = self.config.get_business_object_field_id(busObId, busObFieldName)
                if busObFieldId not in busObFieldIds:
                    busObFieldIds.append(busObFieldId)
            if busObFieldIds:
                kwargs['fields'] = busObFieldIds

        try:
            search_request = SearchResultsRequest(**kwargs)
            api_instance = SearchesApi(self.api_client)
            api_response = api_instance.searches_get_search_results_ad_hoc_v1(search_request)
        except ApiException as e:
            self.log.error('Exception when calling SearchesApi->searches_get_search_results_ad_hoc_v1: %s', e)
            return incidents
        api_response = api_response.to_dict()
        incidents = api_response['business_objects']
        if 'output_format' not in opts:
            return incidents
        if opts['output_format'] not in ['csv', 'text']:
            return incidents

        if 'output_format' in opts and opts['output_format'] in ['csv', 'text']:
            rows = []
            if not busObFields:
                for incident in incidents:
                    if 'fields' not in incident:
                        continue
                    for field in incident['fields']:
                        if 'display_name' not in field:
                            continue
                        if field['display_name'] in busObFields:
                            continue
                        v = self._clean_charset(field['display_name'])
                        busObFields.append(v)

            rows.append(busObFields)
            for incident in incidents:
                row = []
                if 'fields' not in incident:
                    continue
                incident_field_ref = {}
                incident_fields = incident['fields']
                for incident_field in incident_fields:
                    if 'value' not in incident_field:
                        continue
                    if incident_field['value'] == '':
                        continue
                    for k in ['display_name', 'name']:
                        if k not in incident_field:
                            continue
                        if incident_field[k] not in busObFields:
                            continue
                        incident_field_ref[incident_field[k]] = incident_field['value']
                for k in busObFields:
                    if k not in incident_field_ref:
                        row.append('---')
                        continue
                    v = self._clean_charset(incident_field_ref[k])
                    row.append(v)
                rows.append(row)
            return rows

        return incidents

    def add_journal_note(self, opts={}):
        return {'status': 'success'}

    @staticmethod
    def _clean_charset(s):
        s = str(s)
        arr = s.split('\n')
        if len(arr) == 1:
            return s.rstrip()
        for i, v in enumerate(arr):
            arr[i] = v.rstrip()
        return '\n'.join(arr)

    def _get_field_value(self, entry, pair):
        for f in entry:
            if pair[0] not in f or pair[1] not in f:
                continue
            if f[pair[0]] == pair[2]:
                return f[pair[1]]
        return ""

    def _get_csv_columns(self, header, entry):
        """Returns list of column headers.

        Args:
            header: List of business objects.
            entry: List of business objects.

        Returns:
            Returns a CSV row.

        """

        ref_entry = {}
        for f in entry:
            if 'field_id' not in f:
                continue
            ref_entry[f['field_id']] = f

        row = []
        for f in header:
            if 'field_id' not in f:
                continue
            if f['field_id'] not in ref_entry:
                row.append('')
                continue
            if 'value' not in ref_entry[f['field_id']]:
                row.append('')
                continue
            row.append(ref_entry[f['field_id']]['value'])
        return row


    def _get_csv_headers(self, entry, name):
        """Returns list of column headers.

        Args:
            entry: List of objects (dictionaries).
            name: The reference to the key containing column name.

        Returns:
            Returns list of strings.

        """

        headers = []
        for f in entry:
            if 'field_id' not in f:
                continue
            if 'name' not in f:
                headers.append(f['field_id'])
                continue
            headers.append(f[name])
        return headers


    def _serialize_kv(self, entry, pair, opts={}):
        """Converts a list of objects (dict) to list of two-element lists.

        Args:
            entry: List of objects (dictionaries).
            pair: The names of key and values.
            opts: Upstream parameters and options.

        Returns:
            returns a single dict containing the incident information.

        """
        width = None

        if 'terminal_columns' in opts and opts['terminal_columns'] > 0:
            if 'output_format' in opts and opts['output_format'] == 'text':
                width = {}
                width[0] = 30
                width[1] = opts['terminal_columns'] - width[0] - 10

        rows = []
        for f in entry:
            row = []
            for i in [0, 1]:
                if pair[i] in f:
                    v = str(f[pair[i]])
                    if width:
                        v = "\n".join(textwrap.wrap(v, width=width[i]))
                    row.append(v)
                    continue
                row.append('-')
            rows.append(row)
        return rows

    def _authenticate(self):
        ''' Get cached authentication parameters '''
        self.auth_token = self.config.get_auth_token()
        if self.auth_token:
            self.log.debug("Found cached auth token %s", self.auth_token)
            self._authenticated = True
            return
        self.log.debug("Cached auth token not found")
        headers = {
            "Accept-Charset": "utf-8",
            "Accept-Encoding": "gzip, deflate, compress",
            "Accept": "application/json",
            "Content-Type": "application/x-www-form-urlencoded",
            'User-Agent': self.user_agent,
        }
        url = self.config.get_auth_url()
        if not url:
            raise Exception('client', 'authentication URL is empty')
        self.log.debug("Authentication URL: %s", url)
        fields = self.config.get_auth_req_fields()
        if not fields:
            raise Exception('client', 'authentication request fields are empty')
        self.log.debug('Authentication request fields: %s', fields)
        session = requests.Session()
        req = session.post(url, headers=headers, data=fields, verify=False)
        if req.status_code != 200:
            raise Exception('client', '%d: %s, message: %s' % (req.status_code, req.reason, req.text))
        self.log.debug("Authenticator response: %d %s", req.status_code, req.text)
        data = json.loads(req.text)
        if 'access_token' not in data:
            raise Exception('client', 'access_token is not in the response from authenticator')
        self.auth_token = data
        self._authenticated = True
        self.config.save_app_section('token', data)
        return
